<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\CreditCard\CreditCardTransaction;
use Zeedhi\PaymentPlatform\Implementations\MundiPagg\PaymentImpl;

class RechargeMundiPagg extends \Zeedhi\Framework\Controller\Simple {

    protected $instanceManager;
    protected $rechargeService;
    protected $consumerService;
    protected $snackMessage;
    protected $util;
    protected $bancoService;
    protected $vendaOnlineservice;
    protected $smtpService;
    protected $balanceService;

    public function __construct(\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Service\Recharge $rechargeService, \Util\SnackMessage $snackMessage, \Util\Util $util,\Service\Banco $bancoService, \Service\ParaVendaOnline $vendaOnlineservice, \Service\SMTP $smtpService, \Service\Balance $balanceService){
        $this->instanceManager      = $instanceManager;
        $this->rechargeService      = $rechargeService;
        $this->snackMessage         = $snackMessage;
        $this->util                 = $util;
        $this->bancoService         = $bancoService;
        $this->vendaOnlineservice   = $vendaOnlineservice;
        $this->smtpService          = $smtpService;
        $this->balanceService       = $balanceService;
    }

    public function getRechargeGrid(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL = $params[0]['value'];

            $grid = $this->rechargeService->getRechargeGrid($CDFILIAL);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeGridRepository', $grid));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function paymentServiceExists(){
        if (class_exists("\Payment\PaymentService")){
            return true;
        } else {
            throw new \Exception($this->snackMessage->getMessage('9'));
        }
    }

    private function verifyConsumerData($consumerData){

        if (Empty($consumerData['NMRESPCONS'])){
            throw new \Exception($this->snackMessage->getMessage('10'));
        }

        if (Empty($consumerData['NRCPFRESPCON'])){
            throw new \Exception($this->snackMessage->getMessage('11'));
        }

        if (Empty($consumerData['DSEMAILCONS'])){
            throw new \Exception($this->snackMessage->getMessage('29'));
        }

        if (Empty($consumerData['NRTELECONS'])){
            throw new \Exception($this->snackMessage->getMessage('30'));
        }

        $enderecoLabels = array(
            'DSENDECONS',
            'NMPAIS',
            'SGESTADO',
            'NMMUNICIPIO',
            'NRCEPCONS',
            'NMBAIRRO',
            'NRENDECONS'
        );

        foreach ($enderecoLabels as $label){
            if (Empty($consumerData[$label])){
                //Endereço incompleto
                throw new \Exception($this->snackMessage->getMessage('12'));
            }
        }
    }

    public function doBilletRecharge(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL        = $params[0]['value'];
            $CDCLIENTE       = $params[1]['value'];
            $CDCONSUMIDOR    = $params[2]['value'];
            $banco           = $params[3]['value'];
            $valores         = $params[4]['value'];
            $bank            = json_decode($params[5]['value'], true);
            $billetPath      = $params[6]['value'];

            $appUrl          = $params[7]['value'];
            $logoEmpresa     = $params[8]['value'];
            $logoEmpresaInfo = array(
                'appUrl'      => $appUrl,
                'logoEmpresa' => $logoEmpresa
            );

            $filialData      = $this->rechargeService->getFilialData($CDFILIAL);
            $filialEnde      = $this->rechargeService->getFilialEndereco($CDFILIAL);
            $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);

            $NRPEDCRECONS    = $this->setBilletNumber($bank);

            $this->verifyConsumerData($consumerDetails);

            $rechargeValues = json_decode($valores, true);
            $valor = 0.0;

            foreach ($rechargeValues as $value){
                $valor += floatval(str_replace(',', '.', $value['VRSALDCONFAM']));
            }

            $dadosBoleto = array(
                'prazoPagamento'  => $bank['QTDIASVENVND'],
                'valor'           => $valor,
                'nossoNumero'     => $NRPEDCRECONS,
                'numeroDocumento' => $NRPEDCRECONS,
                'codigoCedente'   => $bank['DSINSCEMPVND']
            );

            $dadosCliente = array(
                'nomeCliente'  => $consumerDetails['NMRESPCONS'],
                'endereco1'    => $consumerDetails['DSENDECONS'] . ' ' . $consumerDetails['NMBAIRCONS'] . '' . $consumerDetails['NRCEPCONS'],
                'endereco2'    => $consumerDetails['NMMUNICIPIO'] . '/' . $consumerDetails['SGESTADO']
            );

            $dadosLoja = array(
                'nomeFantasia' => $filialData['NMFILIAL'],
                'siteCliente'  => $filialEnde['DSEMAILFILI'], //PARAM GERAL DSEMAISACVND
                'cpfCnpj'      => $filialData['NRINSJURFILI'],
                'endereco'     => $filialEnde['DSENDEFILI'] . ' ' . $filialEnde['NMBAIRFILI'],
                'cidadeEstado' => $filialEnde['NMMUNICIPIO'] . '/' . $filialEnde['SGESTADO'],
                'razaoSocial'  => $filialData['NMRAZSOCFILI'],
                'convenio'     => $bank['CDBANCOCONVVND'],
                'contrato'     => $bank['CDCONTRATOVND'],
                'conta' => array(
                    'agencia'       => $bank['CDAGENCIAVND'],
                    'digitoAgencia' => $bank['DIGCDAGENCIAVND'],
                    'conta'         => $bank['CDCONTCORR'],
                    'digitoConta'   => $bank['DIGCDCONTCORR']
                )
            );

            $obs = "Credito no cartao do aluno: " . $consumerDetails['NMCONSUMIDOR'];

            if ($this->paymentServiceExists()){
                $paymentService = new \Payment\PaymentService();
                $paymentResponse = $paymentService->generateBillet($bank['CDBANCCNAB'], $dadosBoleto, $dadosCliente, $dadosLoja, $obs, $NRPEDCRECONS);

                if ($paymentResponse['error'] == null){

                    $billetUrl = $NRPEDCRECONS;
                    str_pad($NRPEDCRECONS, 20, 0, STR_PAD_LEFT);

                    $this->rechargeService->insertPendentCredit(
                        $NRPEDCRECONS,
                        floatval(str_replace(',', '.', $valor)),
                        $consumerDetails['CDCLIENTE'],
                        $consumerDetails['CDCONSUMIDOR']
                    );

                    foreach ($rechargeValues as $family){
                        $this->rechargeService->insertPendentCreditFamily(
                            $NRPEDCRECONS,
                            $family['CDFAMILISALD'],
                            floatval(str_replace(',', '.', $family['VRSALDCONFAM']))
                        );
                    }

                    // envia boleto por email
                    $mailSent = false;
                    try {
                        $this->billetEmail($consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $NRPEDCRECONS, $billetPath, $logoEmpresaInfo);
                        $mailSent = true;

                    } catch (\Exception $e){
                        $response->addMessage(new Message($this->snackMessage->getMessage('26'))); // Não foi possível enviar o boleto por email.
                    }

                    if ($mailSent){
                        $response->addMessage(new Message('Foi encaminhado um boleto para o seu e-mail.'));
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeBilletRepository', array('NRPEDCRECONS' => $billetUrl)));
                    }
                }
                else {
                    $response->addMessage(new Message($this->snackMessage->getMessage('26') . $paymentResponse['error']));
                }
            } else {
                $response->addMessage(new Message($this->snackMessage->getMessage('9')));
            }
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function billetEmail($CDCLIENTE, $CDCONSUMIDOR, $NRPEDCRECONS, $billetPath, $logoEmpresaInfo){
        $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);

        $billetUrl          = $billetPath . $NRPEDCRECONS . '.html';
        $emailDestinatario  = $consumerDetails['DSEMAILCONS'];

        $subject    = $this->snackMessage->getMessage('27');
        $sender     = $this->snackMessage->getMessage('14');
        $content    = '<center>'.$this->snackMessage->getMessage('23').'<br>'.$billetUrl.'</center>';

        try {
            $this->smtpService->sendEmail($consumerDetails['NMRESPCONS'], $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('26'));
        }
    }

    public function getSitefPorFilial(Request\Filter $request, Response $response) {
        try {
            $params     = $request->getFilterCriteria()->getConditions();
            $CDFILIAL   = $params[0]['value'];

            $paramsSitef = $this->rechargeService->getSitefPorFilial($CDFILIAL);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetSitefPorFilialRepo', $paramsSitef));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getBankData(Request\Filter $request, Response $response){
        try {
            $params     = $request->getFilterCriteria()->getConditions();
            $CDFILIAL   = $params[0]['value'];
            $bancos     = $this->bancoService->buscaBancos();


            // valida se possui esitef parametrizado
            // procura primeiro por filial depois por geral
            $esitefDetails = $this->rechargeService->getSitefPorFilial($CDFILIAL);
            if (empty($esitefDetails)) {
                $esitefDetails = $this->getEsitefGeral();
            } else {
                if ((empty($esitefDetails[0]['IDLOJAESITEF'])) || (empty($esitefDetails[0]['CDLOJAESITEF']))) {
                    $esitefDetails = $this->getEsitefGeral();
                }
            }

            if (empty($bancos) && empty($esitefDetails)){
                $response->addMessage(new Message($this->snackMessage->getMessage('17')));
            } else {
                foreach ($bancos as &$banco){
                    $banco = $this->mascara($banco, $bancos[0]['CDPICTAGEN'], 'CDAGENCIAVND');
                    $banco = $this->mascara($banco, $bancos[0]['CDPICTCONT'], 'CDCONTCORR');
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('EsitefRepository', $esitefDetails));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BankDataRepository', $bancos));

        } catch (\Exception $e){
             $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function getEsitefGeral() {
        $esitefDetails = $this->rechargeService->getEsitefDetails();
        if (empty($esitefDetails)) {
            $esitefDetails = array(array('array vazio'));
        } else {
            if ((empty($esitefDetails[0]['IDLOJAESITEF'])) || (empty($esitefDetails[0]['CDLOJAESITEF']))) {
                $esitefDetails = array(array('array vazio'));
            }
        }
        return $esitefDetails;
    }

    public function mascara($array, $mascara, $campo){
        $ndigitos = strpos($mascara, '-');

        if (($ndigitos) == false){
            $contaAtual = $array[$campo];
            $array['DIG'.$campo] = "";
        }
        else {
            $contaAtual = $array[$campo];
            $array[$campo] = substr($contaAtual, 0, $ndigitos);
            $array['DIG'.$campo]= substr($contaAtual, ($ndigitos));
        }
        return $array;
    }

    private function setBilletNumber($bank){
        if ($bank['CDBANCCNAB'] === '399'){                               // HSBC
            $NRBOLETO = $this->util->newCode('PEDICREDCONS', 13); // HSBC
        }
        else if ($bank['CDBANCCNAB'] === '033') {                         // Santander
            $NRBOLETO = $this->util->newCode('PEDICREDCONS', 12); // Santander
        }
        else if ($bank['CDBANCCNAB'] === '237') {                         // Bradesco
            $NRBOLETO = $this->util->newCode('PEDICREDCONS', 11); // Bradesco
        }
        else if ($bank['CDBANCCNAB'] === '341') {                         // Itaú
            $NRBOLETO = $this->util->newCode('PEDICREDCONS', 8);  // Itaú
        }
        else if ($bank['CDBANCCNAB'] === '001') {                         // Banco do Brasil
            if (strlen($bank['CDBANCOCONVVND']) == 8){
                $NRBOLETO = $this->util->newCode('PEDICREDCONS', 9);
            }
            else if (strlen($bank['CDBANCOCONVVND']) == 7){
                $NRBOLETO = $this->util->newCode('PEDICREDCONS', 10);
            }
            else if (strlen($bank['CDBANCOCONVVND']) == 6){
                $NRBOLETO = $this->util->newCode('PEDICREDCONS', 5);
            }
        }
        else {
            $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 13); // Default
        }

        return $NRBOLETO;
    }

    public function doCardRecharge(Request\Filter $request, Response $response){
        try {

            $params = $request->getFilterCriteria()->getConditions();

            $valorPedido           = $params[0]['value'];
            $bandeira              = $params[1]['value'];
            $validadeCartao        = $params[2]['value'];
            $numeroCartao          = $params[3]['value'];
            $codigoSegurancaCartao = $params[4]['value'];
            $CDCLIENTE             = $params[5]['value'];
            $CDCONSUMIDOR          = $params[6]['value'];
            $rechargeValues        = $params[7]['value'];
            $CDFILIAL              = $params[8]['value'];
            $appUrl                = $params[9]['value'];
            $logoEmail             = $params[10]['value'];

            if (preg_match("/^4/", $numeroCartao)) $bandeira = "Visa";
            else if (preg_match("/^5/", $numeroCartao)) $bandeira = "Mastercard";
            else if (preg_match("/^3[47]/", $numeroCartao)) $bandeira = "Amex";
            else if (preg_match("/^36/", $numeroCartao)) $bandeira = "Diners";
            else if (preg_match("/^(636368|438935|504175|451416|509048|509067|509049|509069|509050|509074|509068|509040|509045|509051|509046|509066|509047|509042|509052|509043|509064|509040|36297|5067|4576|4011)/", $numeroCartao)) $bandeira = "Elo";
            else {
                $bandeira = null;
                $response->addMessage(new Message("Bandeira não reconhecida. Caso acredite que seu número de cartão de crédito esteja correto, favor entrar em contato com a administração."));
            }

            if ($bandeira){
                $rechargeValues = json_decode($rechargeValues, true);

                $logoEmpresaInfo = array(
                    'appUrl'      => $appUrl,
                    'logoEmail' => $logoEmail
                );

                $online          = $this->vendaOnlineservice->getParamsVendaOnlineFiliGer($CDFILIAL);
                $idLojaSitef     = $online[0]['IDLOJAESITEF'];
                $codigoLojaSitef = $online[0]['CDLOJAESITEF'];

                $NRTRANSACAO = $this->util->newCode('TRANSACAO', 12);
                // Creating consumers list to send e-mail.
                $consumidores = '';
                $consumidorJaAnotado = array();
                foreach ($rechargeValues as $value){
                    if ($consumidores == ''){
                        $consumidores = $value['NMCONSUMIDOR'];
                        array_push($consumidorJaAnotado, $value);
                    }
                    else {
                        foreach ($consumidorJaAnotado as $consumidor){
                            if ($value['CDCONSUMIDOR'] != $consumidor['CDCONSUMIDOR']){
                                $consumidores = $consumidores . ', ' .$value['NMCONSUMIDOR'];
                            }
                        }
                    }
                }

                // Call method to insert into table TRANSACAO
                $transacaoParams = array(
                    'NRTRANSACAO'  => $NRTRANSACAO,
                    'DSTRANSACAO'  => "Transacao iniciada.",
                    'CDCLIENTE'    => $CDCLIENTE,
                    'CDCONSUMIDOR' => $CDCONSUMIDOR,
                    'VRMOVEXTCONS' => $valorPedido
                );
                $this->rechargeService->insertTransacao($transacaoParams);

                $controlArray = array();
                foreach ($rechargeValues as $consumer){
                    if (!($this->consumerAlreadyRegistered($consumer['CDCONSUMIDOR'], $controlArray))){
                        $transactionConsParams = array (
                            'NRTRANSACAO' => $NRTRANSACAO,
                            'CDCLIENTE' => $CDCLIENTE,
                            'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR']
                        );
                        $this->rechargeService->insertTransacaoCons($transactionConsParams);
                    }
                    array_push($controlArray, $consumer['CDCONSUMIDOR']);
                }

                $consumerData = $this->rechargeService->getOwnerName($CDCLIENTE, $CDCONSUMIDOR);
                $consumerData = $consumerData[0];

                //$client = PaymentImpl::factorySandBox("d41a5b8f-ba4b-42b3-bbc6-bc1fb72a3e5c");
                $client = PaymentImpl::factoryProduction("b5b4f2e4-0901-45c9-9105-459b07ab60ac");

                $expMonth   = substr($validadeCartao, 0 , 2);
                $extYear    = substr($validadeCartao, 2);
                $creditCard = new CreditCard(
                    $consumerData['NMRESPCONS'],
                    $numeroCartao,
                    $expMonth,
                    $extYear,
                    $codigoSegurancaCartao,
                    $bandeira
                );

                $creditCardTransaction = new CreditCardTransaction($valorPedido, $NRTRANSACAO, $creditCard);
                $transactionKey        = $client->NonInstantBuyTransaction($creditCardTransaction);

                foreach ($rechargeValues as $family) {
                    $isFirstInsertion = $this->rechargeService->isFirstInsertion($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $currentBalance = $this->rechargeService->getCurrentBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $valueToRecharge = floatval(str_replace(',', '.', $family['VRSALDCONFAM']));

                    // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                    $balance = $this->balanceService->checkBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                    $VRSALDCONEXT = $valueToRecharge;
                    //If it's not first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
                    if(!empty($isFirstInsertion)){
                        //Add VRSALDCONEXT to previous balance
                        if(!empty($currentBalance)){
                            $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                        }
                    }
                    $this->rechargeService->insertExtratoCons($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito No Cash (Cartao)', $valueToRecharge, 'C', $VRSALDCONEXT);

                    //If there is no record in SALDOCONS, we have to create one with current valueToRecharge
                    if (empty($balance)){
                        // Without any balance, the balance can't be digivolved, so we must INSERT ADDITIONAL BALANCES!
                        $this->balanceService->createBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $valueToRecharge);
                    } else {
                        //If there is balance already, we have to update it with the new value
                        $this->balanceService->updateBalance($CDCLIENTE, $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                    }
                }
                $result = true;

                // Update TRANSACAO case success.
                $transacaoParams = array(
                    'NRTRANSACAO' => $NRTRANSACAO,
                    'DSTRANSACAO' => 'Transação feita com sucesso.',
                    'NRNIT'       => $transactionKey
                );
                $this->rechargeService->updateTransacao($transacaoParams);

                // E-mail - The consumerDetails is the same for all the consumers with the same CDCLIENTE, so we don't need to build a foreach to each one.
                $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);
                $codigoCartao    = substr($numeroCartao, 0, 4).' '.substr($numeroCartao, 4, 2).'XX XXXX XX'.substr($numeroCartao, 14);

                // Formating values to send e-mail
                $valorPedido = str_replace(".", "", $valorPedido);
                $valorPedido = substr($valorPedido, 0, (strlen($valorPedido)-2)).','.substr($valorPedido, -2, strlen($valorPedido));
                $temp        = $valorPedido.substr($valorPedido, 2);

                $nomeDestinatario = $consumerDetails['NMRESPCONS'];
                $emailDestinatario = $consumerDetails['DSEMAILCONS'];

                // Creating beautiful e-mail template.
                $subject = $this->snackMessage->getMessage('24');
                $sender  = $this->snackMessage->getMessage('14');
                $content =
                    '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
                    '<b>Nome do(s) Aluno(s): </b>'.$consumidores.'<br>'.
                    '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
                    '<p><b>Valor Total da Recarga: </b>R$ '.$valorPedido.'</p>'.
                    '<p><b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y") .'</p>'
                ;

                try {
                    // Sends the e-mail!
                    $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
                    $mailSent = true;
                } catch (\Exception $e){
                    $mailSent = false;
                }

                /* *** E-MAIL SENT *** */
                $result = true; // Indicates everything was done successfully.
                if ($mailSent) $response->addMessage(new Message('Transação feita com sucesso!'));
                else $response->addMessage(new Message('Transação feita com sucesso!'." ".$this->snackMessage->getMessage('25')));

                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
            }
        } catch (\Exception $e){
            // Something went wrong with the payment.
            $message = substr($e->getMessage(),0,255);
            // Update TRANSACAO case failure.
            $transacaoParams = array(
                'NRTRANSACAO' => $NRTRANSACAO,
                'DSTRANSACAO' => $message,
                'NRNIT' => null
            );
            $this->rechargeService->updateTransacao($transacaoParams);
            $response->addMessage(new Message($message));
            $result = false; // Indicates something went wrong.
            $response->addMessage(new Message($e->getMessage()));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
        }
    }

    //Check if consumer is already registered in TRANSACAOCONS
    public function consumerAlreadyRegistered($consumer, $consumersRegistered){
        foreach ($consumersRegistered as $cons) {
            if($consumer == $cons){
                return true;
            }
        }
        return false;
    }

}