<?php

namespace Odhen\API\Service;

class Fidelidade {

    protected $entityManager;
    protected $databaseUtil;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Util\Database $databaseUtil) {
        $this->entityManager = $entityManager;
        $this->databaseUtil = $databaseUtil;
    }

    public function validaFidelidade($CDIDCONSUMID) {
        try {
            $connection = $this->entityManager->getConnection();
            $params = array('CDIDCONSUMID' => $CDIDCONSUMID);
            $consumidor = $this->entityManager->getConnection()->fetchAssoc(self::DADOS_CONSUMIDOR, $params);
            
            if (!empty($consumidor)) {
                $params = array(
                    'CDCLIENTE'    => $consumidor['CDCLIENTE'],
                    'CDCONSUMIDOR' => $consumidor['CDCONSUMIDOR']
                );
                $beneficios = $this->entityManager->getConnection()->fetchAll(self::BENEFICIOS_CONSUMIDOR, $params);
                $produtos = array();
                foreach ($beneficios as $beneficioAtual) {
                    $validacaoPeriodo = self::validaPeriodoCampanha($beneficioAtual['DTINIVALBENECONS'], $beneficioAtual['DTFIMVALBENECONS']);
                    if ($validacaoPeriodo['error'] == false) {
                        $params = array(
                            'CDCLIENTE' => $consumidor['CDCLIENTE'],
                            'CDCONSUMIDOR' => $consumidor['CDCONSUMIDOR'],
                            'NRSEQBENEFICIO' => $beneficioAtual['NRSEQBENEFICIO'],
                            'CDCAMPANHA' => $beneficioAtual['CDCAMPANHA']
                        );
                        $produtosCampanha = $this->entityManager->getConnection()->fetchAll(self::PRODUTOS_CAMPANHA, $params);

                        $produtos = array_merge($produtos, $produtosCampanha);
                    }
                }
                if (!empty($produtos)) {
                    return array(
                        'error' => false,
                        'produtos' => $produtos,
                        'consumidor' => $consumidor
                    );
                } else {
                    return array(
                        'error' => true,
                        'message' => 'Cartão não possui beneficios disponíveis no sistema.'
                    );
                }
            } else {
                // Cartão não é válido
                return array(
                    'error' => true,
                    'message' => 'Cartão inválido.'
                );
            }
        } catch(\Exception $e) {
            return array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
    }

    const DADOS_CONSUMIDOR = "
        SELECT CO.CDCLIENTE, CO.CDCONSUMIDOR, CO.CDIDCONSUMID
          FROM CONSUMIDOR CO
         WHERE CO.CDIDCONSUMID = :CDIDCONSUMID
    ";
    const BENEFICIOS_CONSUMIDOR = "
        SELECT  MB.CDCAMPANHA, MB.NRSEQBENEFICIO, MB.DTINIVALBENECONS, MB.DTFIMVALBENECONS,
                MB.QTDEBENEFICIO
          FROM MOVBENEFICIOCTR MB
          WHERE MB.CDCLIENTE    = :CDCLIENTE
           AND MB.CDCONSUMIDOR = :CDCONSUMIDOR
           AND QTDEBENEFICIO > 0
    ";


    private function validaPeriodoCampanha($DTINIVALBENECONS, $DTFIMVALBENECONS) {
        try {
            $DTINIVALBENECONS = $this->databaseUtil->convertToDateGeneric('Y-m-d H:i:s.u', $DTINIVALBENECONS);
            $DTFIMVALBENECONS = $this->databaseUtil->convertToDateGeneric('Y-m-d H:i:s.u', $DTFIMVALBENECONS);
            if ($DTINIVALBENECONS > $DTFIMVALBENECONS) {
                return array(
                    'error' => true,
                    'message' => 'Data do início do beneficio maior que a data de término.'
                );
            } elseif ($DTINIVALBENECONS == $DTFIMVALBENECONS) {
                return array(
                    'error' => true,
                    'message' => 'Data do início do beneficio igual a data de término.'
                );
            } else {
                $currentDate = new \DateTime("now");
                if ($currentDate > $DTFIMVALBENECONS) {
                    return array(
                        'error' => true,
                        'message' => 'Cartão fidelidade vencido.'
                    );
                } else {
                    return array(
                        'error' => false
                    );
                }
            }
        } catch(\Exception $e) {
            return array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
    }

    const PRODUTOS_CAMPANHA = "
        SELECT PB.CDPRODUTO, MB.CDCAMPANHA, MB.NRSEQPRODCAMP, MB.NRSEQBENEFICIO,  MB.IDTPDESBENECONS, MB.QTDEBENEFICIO,
               MB.VRDESBENECONS, BF.DSBENEFICIO
        FROM MOVBENEFICIOCTR MB JOIN PRODBENECAMP PB
                                  ON MB.CDCAMPANHA = PB.CDCAMPANHA
                                 AND MB.NRSEQPRODCAMP = PB.NRSEQPRODCAMP
                                 AND MB.NRSEQBENEFICIO = PB.NRSEQBENEFICIO
                                JOIN BENEFICIOFOS BF
                                  ON BF.NRSEQBENEFICIO = PB.NRSEQBENEFICIO
        WHERE MB.CDCLIENTE = :CDCLIENTE
        AND MB.CDCONSUMIDOR = :CDCONSUMIDOR
        AND PB.NRSEQBENEFICIO = :NRSEQBENEFICIO
        AND PB.CDCAMPANHA = :CDCAMPANHA
    ";

    public function insertMovPontoConsAux($campanha, $vrPontoBrinde, $dadosResgate){
        $arrayParams = array(
            'CDFILIAL' => $dadosResgate['dadosComanda']['CDFILIAL'],
            'NRVENDAREST' => $dadosResgate['dadosComanda']['NRVENDAREST'],
            'NRCOMANDA' => $dadosResgate['dadosComanda']['NRCOMANDA'],
            'NRPRODCOMVEN' => '000001',
            'CDCAMPANHA' => $campanha,
            'IDTPMOVCONS' => 'D',
            'VRPONTOBRINDE' => $vrPontoBrinde['VRPONTOBRINDE'],
            'CDLOJA' => $dadosResgate['dadosComanda']['CDLOJA'],
            'NRORG' => 1,
            'IDATIVO' => 'S'
        );
        $this->entityManager->getConnection()->executeQuery(self::INSERT_MOVPONTOCONSAUX, $arrayParams);
    }

    const INSERT_MOVPONTOCONSAUX = "
        INSERT INTO MOVPONTOCONSAUX
        (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDCAMPANHA, IDTPMOVCONS, VRPONTOBRINDE, CDLOJA, NRORG, IDATIVO, DTINCLUSAO, DTULTATU)
        VALUES
        (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDCAMPANHA, :IDTPMOVCONS, :VRPONTOBRINDE, :CDLOJA, :NRORG, :IDATIVO, SYSDATE, SYSDATE)
    ";
}