<?php

namespace Odhen\API\Service;

class Parametros {

    protected $entityManager;
    protected $date;
    protected $precoService;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Util\Date $date, \Odhen\API\Service\Preco $precoService, \Odhen\API\Util\Util $util){
        $this->entityManager = $entityManager;
        $this->date = $date;
        $this->precoService = $precoService;
        $this->util = $util;
    }

    const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';
    const TIPO_GRUPO = '2';
    const TIPO_SUB_GRUPO = 'I';
    const OBS_TIPO_ADICIONAR = 'A';

    public function buscaFiliaisByOperador($CDOPERADOR){
        $params = array(
            ':CDOPERADOR' => $CDOPERADOR
        );
        return $this->entityManager->getConnection()->fetchAll(self::GET_FILIAIS_BY_OPERADOR, $params);
    }

    public function buscaClientePadrao($CDFILIAL){
        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $dadosFilial = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_FILIAL, $params);
        return $dadosFilial[0]['CDCLIENTE'];
    }

    public function carregaDados($CDFILIAL, $CDCAIXA, $CDVENDEDOR, $CDOPERADOR = null){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA
        );
        $dadosCaixa = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_CAIXA, $params);

        $NRCONFTELA = $dadosCaixa[0]['NRCONFTELA'];
        $CDLOJA = $dadosCaixa[0]['CDLOJA'];

        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $dadosFilial = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_FILIAL, $params);

        $CDCLIENTE = $dadosFilial[0]['CDCLIENTE'];

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA
        );
        $dadosLoja = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_LOJA, $params);

        $NRMESAPADRAO = $dadosLoja[0]['NRMESAPADRAO'];

        if ($CDOPERADOR){
        	$dadosVendedor = array(
        		array(
	        		'CDVENDEDOR' => 0,
	        		'NMFANVEN' => 'Vendedor nao registrado.'
        		)
        	);
        }
        else {
	        $params = array(
	            'CDVENDEDOR' => $CDVENDEDOR
	        );
            $dadosVendedor = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_VENDEDOR, $params);

	        $CDOPERADOR = $dadosVendedor[0]['CDOPERADOR'];
        }

        $params = array(
            'CDOPERADOR' => $CDOPERADOR
        );
        $dadosOperador = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_OPERADOR, $params);

        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $dadosParavend = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_PARAVEND, $params);

        $precosIndexadosPorProduto = self::buscaTabelaPadrao($CDFILIAL, $CDCLIENTE, $CDLOJA);
        $precosIndexadosPorProduto = $precosIndexadosPorProduto['precosIndexadosPorProduto'];

        $observacoes = self::trataObservacoes($CDFILIAL, $CDLOJA, $precosIndexadosPorProduto);
        $ambientes = self::montaAmbientes($CDFILIAL, $NRCONFTELA, $NRMESAPADRAO);

        $params = array(
            'CDFILIAL'   => $CDFILIAL,
            'NRCONFTELA' => $NRCONFTELA
        );
        $grupoRecebimentos = $this->entityManager->getConnection()->fetchAll(self::GET_GRUPO_TIPO_RECEBIMENTOS, $params);

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA
        );
        $recebimentos = $this->entityManager->getConnection()->fetchAll(self::GET_TIPO_RECEBIMENTOS, $params);

        $cardapio = self::montaCardapio($CDFILIAL, $NRCONFTELA, $CDLOJA, $CDCLIENTE, $precosIndexadosPorProduto);

        if ($cardapio['error'] == false){
            $cardapio = $cardapio['cardapio'];

            $parametros = array(
                'CDOPERADOR'     => $dadosOperador[0]['CDOPERADOR'],
                'NMOPERADOR'     => $dadosOperador[0]['NMOPERADOR'],
                'CDVENDEDOR'     => $dadosVendedor[0]['CDVENDEDOR'],
                'NMFANVEN'       => $dadosVendedor[0]['NMFANVEN'],
                'NRATRAPADRAO'   => $dadosParavend[0]['NRATRAPADRAO'],
                'CDCAIXA'        => $dadosCaixa[0]['CDCAIXA'],
                'CDLOJA'         => $dadosCaixa[0]['CDLOJA'],
                'IDTPEMISSAOFOS' => $dadosCaixa[0]['IDTPEMISSAOFOS'],
                'NRCONFTELA'     => $dadosCaixa[0]['NRCONFTELA'],
                'IDTPTEF'        => $dadosCaixa[0]['IDTPTEF'],
                'CDTERTEF'       => $dadosCaixa[0]['CDTERTEF'],
                'CDFILIAL'       => $dadosFilial[0]['CDFILIAL'],
                'NMFILIAL'       => $dadosFilial[0]['NMFILIAL'],
                'NRORG'          => $dadosFilial[0]['NRORG'],
                'NRINSJURFILI'   => $dadosFilial[0]['NRINSJURFILI']
            );

            $dados = array(
                'parametros'        => $parametros,
                'cardapio'          => $cardapio,
                'observacoes'       => $observacoes,
                'ambientes'         => $ambientes,
                'recebimentos'      => $recebimentos,
                'grupoRecebimentos' => $grupoRecebimentos
            );

            $result = array(
                'error' => false,
                'dados' => $dados
            );
        }
        else $result = $cardapio;

        return $result;
    }

    public function trataObservacoes($CDFILIAL, $CDLOJA, $precosIndexadosPorProduto) {
        date_default_timezone_set('America/Sao_Paulo');
        $NRDIASEMANABLOQ = date("w") + 1;

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
        );
        if($this->util->databaseIsOracle()){
            $observacoes = $this->entityManager->getConnection()->fetchAll(self::BUSCA_OBSERVACOES_ORACLE, $params);
        } else {
            $observacoes = $this->entityManager->getConnection()->fetchAll(self::BUSCA_OBSERVACOES_SQL, $params);
        }

        foreach ($observacoes as &$obs){
            // Busca preços das observações.
            if (!empty($obs['CDPRODUTO'])) {
                if ($obs['IDCONTROLAOBS'] == self::OBS_TIPO_ADICIONAR){
                    if (!empty($precosIndexadosPorProduto[$obs['CDPRODUTO']])){
                        $obs['VRPRECITEM'] = $precosIndexadosPorProduto[$obs['CDPRODUTO']]['VRPRECITEM'];
                    }
                    else {
                    	$obs['VRPRECITEM'] = 0;
                    }
                }
            }

            /* CALCULATES COMPLEMENTARY COLORS (by Marcus) */
            if (strlen($obs['NRCORSINAL']) === 7){
                $R = hexdec(substr($obs['NRCORSINAL'], 1, 2));
                $G = hexdec(substr($obs['NRCORSINAL'], 3, 2));
                $B = hexdec(substr($obs['NRCORSINAL'], 5, 2));

                /* Squaring the color values. /*/
                $R *= $R;
                $G *= $G;
                $B *= $B;

                /* Note: 65025 = 255 x 255. */
                $Rc = str_pad(dechex(intval(sqrt(65025 - $R))), 2, '0', STR_PAD_LEFT);
                $Gc = str_pad(dechex(intval(sqrt(65025 - $G))), 2, '0', STR_PAD_LEFT);
                $Bc = str_pad(dechex(intval(sqrt(65025 - $B))), 2, '0', STR_PAD_LEFT);

                $obs['NRCORSINALC'] = '#' . $Rc . $Gc . $Bc;
            } else {
                /* Default to black. */
                $obs['NRCORSINALC'] = '#000000';
            }

            $obs['DSENDEIMGOCORR'] = !empty($obs['DSENDEIMGOCORR']) ? self::URL_SERVIDOR_IMAGEM . $obs['DSENDEIMGOCORR'] : null;
        }

        return $observacoes;
    }


    private function montaAmbientes($CDFILIAL, $NRCONFTELA, $NRMESAPADRAO){
        $retornoAmbientes = array();
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRCONFTELA' => $NRCONFTELA
        );
        $ambientes = $this->entityManager->getConnection()->fetchAll(self::SQL_BUSCA_AMBIENTES, $params);

        if (!empty($ambientes)){
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'NRCONFTELA' => $NRCONFTELA,
                'NRMESAPADRAO' => $NRMESAPADRAO
            );
            $mesas = $this->entityManager->getConnection()->fetchAll(self::SQL_BUSCA_MESAS, $params);

            if (!empty($mesas)){
                foreach ($ambientes as $ambienteAtual){
                    $retornoAmbientes[$ambienteAtual['CDIDENTBUTON']] = array(
                        'area' => array(
                            'CDSALA' => $ambienteAtual['CDIDENTBUTON'],
                            'NMSALA' => $ambienteAtual['DSBUTTON'],
                            'NRBUTTON' => $ambienteAtual['NRBUTTON']
                        ),
                        'mesas' => array()
                    );

                    foreach ($mesas as $mesaAtual){
                        if ($mesaAtual['NRPGCONFTAUX'] == $ambienteAtual['NRPGCONFTELA'] && $mesaAtual['NRBUTTONAUX'] == $ambienteAtual['NRBUTTON']){
                            $retornoAmbientes[$ambienteAtual['CDIDENTBUTON']]['mesas'][$mesaAtual['CDIDENTBUTON']] = array(
                                'NRMESA' => $mesaAtual['CDIDENTBUTON'],
                                'NMMESA' => $mesaAtual['DSBUTTON']
                            );
                        }
                    }
                }
            }
        }

        return $retornoAmbientes;
    }

    public function montaCardapio($CDFILIAL, $NRCONFTELA, $CDLOJA, $CDCLIENTE, $precosIndexadosPorProduto, $CDCONSUMIDOR = null) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'NRCONFTELA' => $NRCONFTELA
        );
        $grupoProdutos = $this->entityManager->getConnection()->fetchAll(self::BUSCA_GRUPO_PRODUTOS, $params);

        if (!empty($grupoProdutos)){
            $subGrupoProdutos = $this->entityManager->getConnection()->fetchAll(self::BUSCA_SUBGRUPO_PRODUTOS, $params);

            date_default_timezone_set('America/Sao_Paulo');
            $NRDIASEMANABLOQ = date("w") + 1;

            $params = array(
                'CDFILIAL'        => $CDFILIAL,
                'CDLOJA'          => $CDLOJA,
                'NRCONFTELA'      => $NRCONFTELA,
                'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
            );
            if ($this->util->databaseIsOracle()) {
            	$produtos = $this->entityManager->getConnection()->fetchAll(self::BUSCA_PRODUTOS_ORACLE, $params);
            } else {
            	$produtos = $this->entityManager->getConnection()->fetchAll(self::BUSCA_PRODUTOS_SQL, $params);
            }

            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRDIASEMANABLOQ' => $NRDIASEMANABLOQ
            );
            $observacoesProduto = $this->entityManager->getConnection()->fetchAll(self::BUSCA_OBSERVACOES_PRODUTO, $params);

            $observacoesIndexadasPorProduto = array();
            foreach ($observacoesProduto as $obsAtual) {
                if (!(array_key_exists($obsAtual['CDPRODUTO'], $observacoesIndexadasPorProduto))) {
                    $observacoesIndexadasPorProduto[$obsAtual['CDPRODUTO']] = array();
                }
                array_push($observacoesIndexadasPorProduto[$obsAtual['CDPRODUTO']], $obsAtual['CDOCORR']);
            }

            $arrayProdutos = self::montaArrayProdutos($grupoProdutos, $subGrupoProdutos, $produtos,
                            $precosIndexadosPorProduto, $observacoesIndexadasPorProduto, $CDLOJA, $CDFILIAL, $NRCONFTELA,
                            $CDCLIENTE, $CDCONSUMIDOR);

            if (!empty($arrayProdutos["error"])){
                $result = $arrayProdutos;
            }
            else {
                $result = array(
                    'error' => false,
                    'cardapio' => $arrayProdutos
                );
            }
        }
        else {
            $result = array(
                'error' => true,
                'message' => 'Não há grupo de produtos na configuração de tela do caixa.'
            );
        }
        return $result;
    }

    public function buscaTabelaPadrao($CDFILIAL, $CDCLIENTE, $CDLOJA){
        $tabelaPadrao = $this->precoService->buscaTabelaVigente($CDFILIAL, $CDLOJA, $CDCLIENTE);
        if ($tabelaPadrao['error'] == false){
            $params = array(
                'CDFILIAL'    => $tabelaPadrao['CDFILIAL'],
                'CDTABEPREC'  => $tabelaPadrao['CDTABEPREC'],
                'DTINIVGPREC' => $tabelaPadrao['DTINIVGPREC']
            );

            if ($this->util->databaseIsOracle()) {
                $precos = $this->entityManager->getConnection()->fetchAll(self::ORACLE_PRECOS, $params);
            } else {
                $precos = $this->entityManager->getConnection()->fetchAll(self::SQL_PRECOS, $params);
            }

            $precosDia = self::buscaPrecosDia($tabelaPadrao['CDFILIAL'], $tabelaPadrao['CDTABEPREC'], $tabelaPadrao['DTINIVGPREC']);

            if (!empty($precos)) {
                $precosIndexadosPorProduto = array();
                foreach ($precos as $produto) {
                    $precosIndexadosPorProduto[$produto['CDPRODUTO']] = $produto;
                }

                if (!empty($precosDia)) {
                    foreach ($precosDia as $produtoDia) {
                        $codProdutoDia = $produtoDia['CDPRODUTO'];
                        if (isset($precosIndexadosPorProduto[$codProdutoDia]) && $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] !== null) {
                            if ($produtoDia['IDPERVALORPR'] == 'V') {
                                if ($produtoDia['IDDESCACREPR'] == 'D') {
                                    //Desconto por valor
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] - $produtoDia['VRPRECODIA'];
                                } else {
                                    //Acrescimo por valor
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] + $produtoDia['VRPRECODIA'];
                                }
                            } else {
                                if ($produtoDia['IDDESCACREPR'] == 'D') {
                                    //Desconto por porcentagem
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] * (1 - $produtoDia['VRPRECODIA']/100);
                                } else {
                                    //Acrescimo por porcentagem
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] * (1 + ($produtoDia['VRPRECODIA']/100));
                                }
                            }

                            if ($precoFinal > 0) {
                                $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] = $precoFinal;
                            }
                        }
                    }
                }

                $result = array(
                    'error' => false,
                    'precosIndexadosPorProduto' => $precosIndexadosPorProduto
                );
            } else {
                $result = array(
                    'error' => true,
                    'message' => 'Não há produtos para a tabela de preços vigente.'
                );
            }
        } else {
            $result = $tabelaPadrao;
        }
        return $result;
    }

    private function buscaPrecosDia($CDFILIAL, $CDTABEPREC, $DTINIVGPREC) {
        $dataAtual   = $this->date->getDataAtual();
        $hora        = $dataAtual->format("H:i:s");
        $hora        = str_replace(":", "", $hora);
        $hora        = substr($hora, 0, 4);
        // $hora = '1800';

        // Expected: 1 (for Sunday) through 7 (for Saturday)
        // PHP returns: 1 (for Monday) through 7 (for Sunday)
        // So, this method changes the index for each day, changing from the forat of return
        // used by PHP to Delphi format.
        $dia = (((int)$dataAtual->format("N")) % 7) + 1;

        // @toDo: Fix consumer type for different prices
        $params = array(
            'CDFILIAL'    => $CDFILIAL,
            'CDTABEPREC'  => $CDTABEPREC,
            'DTINIVGPREC' => $DTINIVGPREC,
            'HORA'        => $hora,
            'NRDIASEMANPR' => (string) $dia,
            'CDTIPOCONSPD' => 'T'
        );

        if ($this->util->databaseIsOracle()) {
            $precosDia = $this->entityManager->getConnection()->fetchAll(self::ORACLE_PRECOS_DIA, $params);
        } else {
            $precosDia = $this->entityManager->getConnection()->fetchAll(self::SQL_PRECOS_DIA, $params);
        }

        return $precosDia;
    }

    private function validaTabela($CDFILIAL, $CDTABEPREC){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDTABEPREC' => $CDTABEPREC,
            'DATE' => date("d/m/Y")
        );
        $vigenciaTabela = $this->entityManager->getConnection()->fetchAll(self::SQL_VAL_TABE, $params);
        return !empty($vigenciaTabela);
    }

    private function montaArrayProdutos($grupoProdutos, $subGrupoProdutos, $produtos,
                                        $precosIndexadosPorProduto, $observacoesIndexadasPorProduto,
                                        $CDLOJA, $CDFILIAL, $NRCONFTELA, $CDCLIENTE, $CDCONSUMIDOR){

        $arrayRetorno = array();

        // Processa os grupos.
        foreach ($grupoProdutos as $grupoAtual){
            $codGrupo = $grupoAtual['NRPGCONFTELA'] . $grupoAtual['NRBUTTON'];

            $arrayRetorno[$codGrupo] = array(
                'grupo' => array(
                    'CODIGO' => $codGrupo,
                    'DESC' => $grupoAtual['DSBUTTON'],
                    'COLOR' => $grupoAtual['NRCOLORBACK'],
                    'DSENDEIMG' => !empty($grupoAtual['DSENDEIMG']) ? self::URL_SERVIDOR_IMAGEM . $grupoAtual['DSENDEIMG'] : null,
                    'NRBUTTON' => $grupoAtual['NRBUTTON']
                ),
                'produtos' => array()
            );
        }

        // Processa os subgrupos.
        foreach ($subGrupoProdutos as $subGrupoAtual){
            $codGrupo = $subGrupoAtual['NRPGCONFTAUX'] . $subGrupoAtual['NRBUTTONAUX'];
            $codSubGrupo = $subGrupoAtual['NRPGCONFTELA'] . $subGrupoAtual['NRBUTTON'];

            $arrayRetorno[$codGrupo]['subgrupos'][$codSubGrupo] = array(
                'subgrupo' => array(
                    'CODIGO' => $codSubGrupo,
                    'DESC' => $subGrupoAtual['DSBUTTON'],
                    'COLOR' => $subGrupoAtual['NRCOLORBACK']
                ),
                'produtos' => array()
            );
        }

        // Processa os produtos.
        foreach ($produtos as $key => $produtoAtual){
            // Define preço do produto.
            if (array_key_exists($produtoAtual['CDPRODUTO'], $precosIndexadosPorProduto)){
                $doPreco = floatval($precosIndexadosPorProduto[$produtoAtual['CDPRODUTO']]['VRPRECITEM']);
            } else {
                $preco =  $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $produtoAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
                if ($preco['retirarProduto']) unset($produtos[$key]);
                if ($preco['error']) continue;
                else $doPreco = $preco['PRECO'];
            }

            // Define as impressoras do produto.
            $params = array(
                'CDFILIAL' => $produtoAtual['CDFILIAL'],
                'CDLOJA' => $CDLOJA,
                'CDPRODUTO' => $produtoAtual['CDPRODUTO'],
                'NRCONFTELA' => $produtoAtual['NRCONFTELA']
            );
            $impressorasProduto = $this->entityManager->getConnection()->fetchAll(self::SQL_GET_IMP_PRODUTOS, $params);

            // Trata promoção inteligente.
            $gruposPromocao = array();
            if ($produtoAtual['IDTIPOCOMPPROD'] == '3'){
                $params = array(
                    'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON'],
                    'CDFILIAL' => $CDFILIAL
                );
                $composicaoPadrao = $this->entityManager->getConnection()->fetchAll(self::SQL_GRUPOS_PROMO_INT_FILI, $params);

                if (count($composicaoPadrao) > 0){
                    $usaComposicaoPadrao = false;
                    $gruposComposicao = $composicaoPadrao;
                }
                else {
                    $usaComposicaoPadrao = true;
                    $params = array(
                        'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON']
                    );
                    $gruposComposicao = $this->entityManager->getConnection()->fetchAll(self::SQL_GRUPOS_PROMO_INT, $params);
                }

                if (!empty($gruposComposicao)){
                    foreach ($gruposComposicao as $smartGroup){
                        $gruposPromocao[$smartGroup['CDGRUPROMOC']] = array(
                            'grupo' => array(),
                            'produtos' => array()
                        );

                        $gruposPromocao[$smartGroup['CDGRUPROMOC']]['grupo'] = array(
                            'CDGRUPROMOC' => $smartGroup['CDGRUPROMOC'],
                            'NMGRUPROMOC' => $smartGroup['NMGRUPROMOC'],
                            'QTPRGRUPPROMOC' => intval($smartGroup['QTPRGRUPPROMOC'])
                        );

                        date_default_timezone_set('America/Sao_Paulo');
                        $NRDIASEMANABLOQ = date("w") + 1;

                        if ($usaComposicaoPadrao){
                            $params = array(
                                'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON'],
                                'CDGRUPROMOC' => $smartGroup['CDGRUPROMOC'],
                                'CDFILIAL' => $CDFILIAL,
                                'CDLOJA' => $CDLOJA,
                                'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
                            );
                            if($this->util->databaseIsOracle()){
                                $produtosComposicao = $this->entityManager->getConnection()->fetchAll(self::ORACLE_PRODUTOS_PROMO_INT, $params);
                            } else {
                                $produtosComposicao = $this->entityManager->getConnection()->fetchAll(self::SQL_PRODUTOS_PROMO_INT, $params);
                            }
                        }
                        else {
                            $params = array(
                                'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON'],
                                'CDGRUPROMOC'  => $smartGroup['CDGRUPROMOC'],
                                'CDFILIAL' => $CDFILIAL,
                                'CDLOJA' => $CDLOJA,
                                'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
                            );
                            if($this->util->databaseIsOracle()){
                                $produtosComposicao = $this->entityManager->getConnection()->fetchAll(self::ORACLE_PRODUTOS_PROMO_INT_FILI, $params);
                            } else {
                                $produtosComposicao = $this->entityManager->getConnection()->fetchAll(self::SQL_PRODUTOS_PROMO_INT_FILI, $params);
                            }
                        }

                        $doPrecoPromoc = '';

                        if (!empty($produtosComposicao)){
                            foreach ($produtosComposicao as $produtoComposicaoAtual) {
                                if (!empty($precosIndexadosPorProduto[$produtoAtual['CDIDENTBUTON']]['VRPRECITEM'])){
                                    if (!empty($precosIndexadosPorProduto[$produtoComposicaoAtual['CDPRODUTO']]['VRPRECITEM'])){
                                        $doPrecoPromoc = $precosIndexadosPorProduto[$produtoComposicaoAtual['CDPRODUTO']]['VRPRECITEM'];
                                    }
                                }
                                else $doPrecoPromoc = 0;

                                $params = array(
                                    'CDFILIAL' => $CDFILIAL,
                                    'CDLOJA' => $CDLOJA,
                                    'CDPRODUTO' => $produtoComposicaoAtual['CDPRODUTO'],
                                    'NRCONFTELA' => $NRCONFTELA
                                );
                                $promoPrinter = $this->entityManager->getConnection()->fetchAll(self::SQL_GET_IMP_PRODUTOS, $params);

                                $gruposPromocao[$smartGroup['CDGRUPROMOC']]['produtos'][$produtoComposicaoAtual['CDPRODUTO']] = array(
                                    'CDPRODUTO' => $produtoComposicaoAtual['CDPRODUTO'],
                                    'IDIMPPRODUTO' => $produtoComposicaoAtual['IDIMPPRODUTO'],
                                    'IDAPLICADESCPR' => $produtoComposicaoAtual['IDAPLICADESCPR'],
                                    'IDPERVALORDES' => $produtoComposicaoAtual['IDPERVALORDES'],
                                    'NMPRODUTO' => empty($produtoComposicaoAtual['DSAPELIDOMOB']) ? $produtoComposicaoAtual['NMPRODUTO'] : $produtoComposicaoAtual['DSAPELIDOMOB'],
                                    'VRDESPRODPROMOC' => $produtoComposicaoAtual['VRDESPRODPROMOC'],
                                    'VRPRECITEM' => $doPrecoPromoc,
                                    'OBSERVACOES' => array_key_exists($produtoComposicaoAtual['CDPRODUTO'], $observacoesIndexadasPorProduto) ? $observacoesIndexadasPorProduto[$produtoComposicaoAtual['CDPRODUTO']] : [],
                                    'IDPRODBLOQ' => $produtoComposicaoAtual['IDPRODBLOQ'],
                                    'IMPRESSORAS' => $promoPrinter,
                                    'DSPRODVENDA' => $produtoComposicaoAtual['DSPRODVENDA'],
                                    'DSADICPROD' => $produtoComposicaoAtual['DSADICPROD'],
                                    'DSENDEIMGPROMO' => !empty($produtoComposicaoAtual['DSENDEIMGPROMO']) ? self::URL_SERVIDOR_IMAGEM . $produtoComposicaoAtual['DSENDEIMGPROMO'] : null,
                                	'NRORDPROMOPR' => $produtoComposicaoAtual['NRORDPROMOPR'] ? $produtoComposicaoAtual['NRORDPROMOPR'] : 99
                                );
                            }
                        }
                    }
                }
            }

            // Formatação final.
            $produtoConstruido = array(
                'CDARVPROD' => $produtoAtual['CDARVPROD'],
                'CDBARPRODUTO' => $produtoAtual['CDBARPRODUTO'],
                'CDPRODINTE' => $produtoAtual['CDPRODINTE'],
                'CDPRODUTO' => $produtoAtual['CDIDENTBUTON'],
                'NMPRODUTO' => $produtoAtual['DSBUTTON'],
                'DTFINVGPROMOC' => $produtoAtual['DTFINVGPROMOC'],
                'DTINIVGPROMOC' => $produtoAtual['DTINIVGPROMOC'],
                'GRUPOS' => $gruposPromocao,
                'OBSERVACOES' => array_key_exists($produtoAtual['CDIDENTBUTON'], $observacoesIndexadasPorProduto) ? $observacoesIndexadasPorProduto[$produtoAtual['CDIDENTBUTON']] : [],
                'IMPRESSORAS'  => $impressorasProduto,
                'IDIMPPRODUTO' => $produtoAtual['IDIMPPRODUTO'],
                'IDPESAPROD' => $produtoAtual['IDPESAPROD'],
                'IDTIPOCOMPPROD' => $produtoAtual['IDTIPOCOMPPROD'],
                'IDTIPOPROD' => $produtoAtual['IDTIPOPROD'],
                'VRPRECITEM' => $doPreco,
                'IDPRODBLOQ' => $produtoAtual['IDPRODBLOQ'],
                'NRCOLORBACK' => $produtoAtual['NRCOLORBACK'],
                'IDCONTROLAREFIL' => $produtoAtual['IDCONTROLAREFIL'],
                'DSENDEIMG' => !empty($produtoAtual['DSENDEIMG']) ? self::URL_SERVIDOR_IMAGEM . $produtoAtual['DSENDEIMG'] : null,
                'DSPRODVENDA' => $produtoAtual['DSPRODVENDA'],
                'DSADICPROD' => $produtoAtual['DSADICPROD'],
                'NMGRUPO' => $arrayRetorno[$produtoAtual['NRPGCONFTAUX'] . $produtoAtual['NRBUTTONAUX']]['grupo']['DESC'],
                'CDBARPRODUTO' => $produtoAtual['CDBARPRODUTO']
            );

            // Insere o produto no grupo ou subgrupo.
            if ($produtoAtual['IDTPBUTONAUX'] == self::TIPO_GRUPO) {
                $codGrupo = $produtoAtual['NRPGCONFTAUX'] . $produtoAtual['NRBUTTONAUX'];
                $arrayRetorno[$codGrupo]['produtos'][$produtoAtual['CDIDENTBUTON']] = $produtoConstruido;
            }
            else if ($produtoAtual['IDTPBUTONAUX'] == self::TIPO_SUB_GRUPO) {
                $codSubGrupo = $produtoAtual['NRPGCONFTAUX'] . $produtoAtual['NRBUTTONAUX'];
                $codGrupo = $produtoAtual['NRPGCONFTAUX2'] . $produtoAtual['NRBUTTONAUX2'];
                $arrayRetorno[$codGrupo]['subgrupos'][$codSubGrupo]['produtos'][$produtoAtual['CDIDENTBUTON']] = $produtoConstruido;
            }
        }

        // Busca produtos combinados.
        date_default_timezone_set('America/Sao_Paulo');
        $NRDIASEMANABLOQ = date("w") + 1;

        $params = array(
            'CDFILIAL'        => $CDFILIAL,
            'CDLOJA'          => $CDLOJA,
            'NRCONFTELA'      => $NRCONFTELA,
            'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
        );
        $produtosCombo = $this->entityManager->getConnection()->fetchAll(self::GET_COMBO_PRODUCTS, $params);

        // Processa os produtos combinados.
        if (sizeof($produtosCombo) > 0){
            $formattedCombos = array();

            $i = 0; // Identificador do produto pai.
            foreach ($produtosCombo as $combo){
                // Busca a composição do produto combinado.
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'NRCONFTELA' => $NRCONFTELA,
                    'IDTIPOPROD' => intval($combo['NRBUTTON']),
                    'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
                );
                if ($this->util->databaseIsOracle()){
                    $comboChildProducts = $this->entityManager->getConnection()->fetchAll(self::BUSCA_PRODUTOS_COMBO_ORACLE, $params);
                } else {
                    $comboChildProducts = $this->entityManager->getConnection()->fetchAll(self::BUSCA_PRODUTOS_COMBO_SQL, $params);
                }

                // Define preços e observações.
                foreach ($comboChildProducts as &$childProduct){
                    if (!empty($precosIndexadosPorProduto[$childProduct['CDIDENTBUTON']]['VRPRECITEM'])){
                        $childProduct['VRPRECITEM'] = $precosIndexadosPorProduto[$childProduct['CDIDENTBUTON']]['VRPRECITEM'];
                    }
                    else {
                        $childProduct['VRPRECITEM'] = 0;
                    }
                    $childProduct['OBSERVACOES'] = array_key_exists($childProduct['CDIDENTBUTON'], $observacoesIndexadasPorProduto) ? $observacoesIndexadasPorProduto[$childProduct['CDIDENTBUTON']] : array();
                }

                // Container necessário do produto combinado.
                $comboCode = str_pad($i++, 5, '0');
                $standardGroup = array(
                    $comboCode => array(
                        'grupo' => array(
                            'CDGRUPROMOC' => $comboCode,
                            'NMGRUPROMOC' => '',
                            'QTPRGRUPPROMOC' => $combo['NRLIMPRODCOM']
                        ),
                        'produtos' => $comboChildProducts // Composição (produtos filhos).
                    )
                );

                // Formatação do produto combinado.
                $preparedCombo = array(
                    'CDARVPROD' => $combo['CDARVPROD'],
                    'CDBARPRODUTO' => $combo['CDBARPRODUTO'],
                    'CDPRODINTE' => $combo['CDPRODINTE'],
                    'CDPRODUTO' => $comboCode,
                    'NMPRODUTO' => $combo['DSBUTTON'],
                    'DTFINVGPROMOC' => $combo['DTFINVGPROMOC'],
                    'DTINIVGPROMOC' => $combo['DTINIVGPROMOC'],
                    'GRUPOS' => $standardGroup, // Itens do produto combinado.
                    'OBSERVACOES' => array(),
                    'IMPRESSORAS'  => array(),
                    'IDIMPPRODUTO' => '2', // Produto pai nunca será impresso.
                    'IDPESAPROD' => $combo['IDPESAPROD'],
                    'IDTIPOCOMPPROD' => 'C',
                    'IDTIPOPROD' => $combo['IDTIPOPROD'],
                    'IDTIPCOBRA' => $combo['IDTIPCOBRA'],
                    'VRPRECITEM' => 0.01,
                    'IDPRODBLOQ' => $combo['IDPRODBLOQ'],
                    'NRCOLORBACK' => $combo['NRCOLORBACK'],
                    'IDCONTROLAREFIL' => $combo['IDCONTROLAREFIL'],
                    'DSENDEIMG' => null,
                    'DSPRODVENDA' => $combo['DSPRODVENDA'],
                    'DSADICPROD' => null,
                    'NMGRUPO' => 'COMBOS'
                );
                array_push($formattedCombos, $preparedCombo);
            }

            // Cria um grupo para os produtos combinados.
            if (!empty($formattedCombos)){
                $arrayRetorno['X'] = array(
                    'grupo' => array(
                        'CODIGO' => 'X',
                        'DESC' => 'COMBOS',
                        'COLOR' => '8421376.000',
                        'DSENDEIMG' => null,
                        'NRBUTTON' => null
                    ),
                    'produtos' => $formattedCombos
                );
            }
        }

        // Busca todos os grupos promocionais presentes no cardapio com preço atualizado.
        $gruposPromocionais = self::buscaGruposPromocionais($arrayRetorno, $CDFILIAL, $CDCLIENTE, $CDLOJA, $CDCONSUMIDOR);
        self::atualizaPrecoGrupPromoc($arrayRetorno, $gruposPromocionais);

        return $arrayRetorno;
    }

    private function buscaGruposPromocionais($cardapio, $CDFILIAL, $CDCLIENTE, $CDLOJA, $CDCONSUMIDOR) {
    	$gruposPromocionais = array();

    	foreach ($cardapio as &$grupoAtual) {
    		if (!empty($grupoAtual['subgrupos'])) {
    			foreach ($grupoAtual['subgrupos'] as $subGrupoAtual) {
    				foreach ($subGrupoAtual['produtos'] as $produtoSubGrupoAtual) {
    					if ($produtoSubGrupoAtual['IDTIPOCOMPPROD'] == '3') {
    						foreach ($produtoSubGrupoAtual['GRUPOS'] as $chaveGrupo => &$grupoPromocAtual) {
			    				$chaveExiste = array_key_exists($chaveGrupo, $gruposPromocionais);
			    				if (!$chaveExiste) {
			    					foreach ($grupoPromocAtual['produtos'] as &$prodGrupoPromocAtual) {
			    						$preco =  $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $prodGrupoPromocAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
	                                    $prodGrupoPromocAtual['VRPRECITEM'] = $preco['PRECO'];
			    					}
			    					$gruposPromocionais[$chaveGrupo] = $grupoPromocAtual;
			    				}
			    			}
    					}
    				}
    			}
    		}
            else {
    			foreach ($grupoAtual['produtos'] as &$produtoAtual) {
		    		if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
		    			// produto é combo
		    			foreach ($produtoAtual['GRUPOS'] as $chaveGrupo => &$grupoPromocAtual) {
		    				// verifica se grupo promocional já foi inserido no array
		    				$chaveExiste = array_key_exists($chaveGrupo, $gruposPromocionais);
		    				if (!$chaveExiste) {
		    					foreach ($grupoPromocAtual['produtos'] as &$prodGrupoPromocAtual) {
		    						// carrega preço dos produtos do grupo promocional corretamente
		    						$preco =  $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $prodGrupoPromocAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
                                    $prodGrupoPromocAtual['VRPRECITEM'] = $preco['PRECO'];
		    					}

		    					$gruposPromocionais[$chaveGrupo] = $grupoPromocAtual;
		    				}
		    			}
		    		}
    			}
    		}
    	}

    	return $gruposPromocionais;
    }

    private function atualizaPrecoGrupPromoc(&$cardapio, $gruposPromocionais) {
    	foreach ($cardapio as &$grupoAtual) {
            if(isset($grupoAtual['produtos'])) {
                foreach ($grupoAtual['produtos'] as &$produtoAtual) {
                    if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
                        // produto é combo
                        foreach ($produtoAtual['GRUPOS'] as $chaveGrupo => &$grupoPromocAtual) {
                            // percorre produtos do grupo promocional e atualiza preços
                            foreach ($grupoPromocAtual['produtos'] as $chaveProduto => &$prodGrupoPromocAtual) {
                                $prodGrupoPromocAtual['VRPRECITEM'] = $gruposPromocionais[$chaveGrupo]['produtos'][$chaveProduto]['VRPRECITEM'];
                            }
                        }
                    }
                }
            }
		}
    }

    const GET_FILIAIS_BY_OPERADOR = "
        SELECT DISTINCT  FL.NMFILIAL,'' AS NMLOJA, C.CDFILIAL||'_'||C.CDLOJA  AS FILIPAI,
               F.CDOPERADOR, C.CDFILIAL AS PARENT , F.CDFILIAL, C.CDLOJA,
               'Loja:'|| C.CDLOJA||' - '||LJ.NMLOJA  AS DESCRICAO,
               '' AS FILILOJA, '' AS CDCAIXA, '' AS IDTPTEF
          FROM CAIXA C JOIN FILIOPER F
                         ON C.CDFILIAL = F.CDFILIAL
                        AND F.NRORG = C.NRORG
                       JOIN FILIAL FL
                         ON F.CDFILIAL = FL.CDFILIAL
                        AND F.NRORG = FL.NRORG
                       JOIN LOJA LJ
                         ON C.CDLOJA= LJ.CDLOJA
                        AND C.CDFILIAL = LJ.CDFILIAL
                        AND C.NRORG = LJ.NRORG
                       JOIN OPERADOR O
                         ON F.CDOPERADOR = O.CDOPERADOR
                        AND O.NRORGTRAB = F.NRORG
         WHERE F.CDOPERADOR = :CDOPERADOR
           AND LJ.IDATIVO = 'S'
           AND C.IDATIVO = 'S'

        UNION

        ---PAI----
        SELECT F.NMFILIAL, '' AS NMLOJA, F.CDFILIAL AS FILIPAI, FL.CDOPERADOR, '' AS PARENT,
               F.CDFILIAL AS FILIPAI, '' AS CDLOJA,  'Filial:'||F.CDFILIAL||' - '||F.NMFILIAL AS DESCRICAO,
               '' AS FILILOJA, '' AS CDCAIXA, '' AS IDTPTEF
          FROM FILIOPER FL JOIN FILIAL F
                             ON F.CDFILIAL = FL.CDFILIAL
                            AND F.NRORG = FL.NRORG
                           JOIN LOJA L
                             ON FL.CDFILIAL = L.CDFILIAL
                            AND FL.NRORG = L.NRORG
                           JOIN CAIXA C
                             ON FL.CDFILIAL = C.CDFILIAL
                            AND FL.NRORG = C.NRORG
                           JOIN OPERADOR O
                             ON FL.CDOPERADOR = O.CDOPERADOR
                            AND O.NRORGTRAB = F.NRORG
         WHERE FL.CDOPERADOR = :CDOPERADOR
           AND F.IDATIVO = 'S'

        UNION

        --- PDV ----
        SELECT DISTINCT  FL.NMFILIAL AS NMFILIAL, LJ.NMLOJA AS NMLOJA,
               C.CDFILIAL||'_'||C.CDLOJA||'_'||C.CDCAIXA  AS FILIPAI, F.CDOPERADOR,
               C.CDFILIAL||'_'||C.CDLOJA AS PARENT , F.CDFILIAL,C.CDLOJA AS CDLOJA,
               'PDV:'|| C.CDCAIXA||' - '||C.NMCAIXA AS DESCRICAO,
               'Filial:'||F.CDFILIAL||' - '||FL.NMFILIAL||'  Loja:'||C.CDLOJA||'    PDV:'||C.CDCAIXA||' - '||C.NMCAIXA AS FILILOJA,
               C.CDCAIXA AS CDCAIXA, C.IDTPTEF AS IDTPTEF
          FROM CAIXA C JOIN FILIOPER F
                         ON C.CDFILIAL = F.CDFILIAL
                        AND C.NRORG = F.NRORG
                       JOIN FILIAL FL
                         ON F.CDFILIAL = FL.CDFILIAL
                        AND FL.NRORG = F.NRORG
                       JOIN LOJA LJ
                         ON C.CDLOJA= LJ.CDLOJA
                        AND C.CDFILIAL = LJ.CDFILIAL
                        AND C.NRORG = LJ.NRORG
                       JOIN OPERADOR O
                         ON F.CDOPERADOR = O.CDOPERADOR
                        AND O.NRORGTRAB = F.NRORG
         WHERE F.CDOPERADOR = :CDOPERADOR
           AND C.IDATIVO = 'S'
    ";

    const GET_GRUPO_TIPO_RECEBIMENTOS = "
        SELECT
            IT.NRBUTTON, IT.DSBUTTON, IT.NRCOLORBACK
        FROM
            ITMENUCONFTE IT
        WHERE
            (IT.CDFILIAL   = :CDFILIAL  ) AND
            (NRCONFTELA    = :NRCONFTELA) AND
            (IT.IDTPBUTTON = '6')         AND
            (EXISTS
                (SELECT 1
                FROM ITMENUCONFTE
                WHERE (CDFILIAL   = :CDFILIAL  ) AND
                    (NRCONFTELA   = :NRCONFTELA) AND
                    (IDTPBUTTON   = '5')         AND
                    (IDTPBUTONAUX = '6')         AND
                    (NRBUTTONAUX = IT.NRBUTTON)
                )
            )
        ORDER BY NRBUTTON
    ";

    const GET_TIPO_RECEBIMENTOS = "
        SELECT IT.DSBUTTON, TR.CDTIPORECE, TR.IDTIPORECE, TR.IDMOSTRARECE,
            IT.NRBUTTON, IT.NRCOLORBACK, IT.NRBUTTONAUX, TR.IDSANGRIAAUTO
          FROM ITMENUCONFTE IT, TIPORECE TR
         WHERE IT.CDFILIAL   = :CDFILIAL
           AND IT.NRCONFTELA = (SELECT NRCONFTELA
                                  FROM CAIXA
                                 WHERE CDFILIAL = :CDFILIAL
                                   AND CDCAIXA = :CDCAIXA)
           AND IT.IDTPBUTTON = '5'
           AND IT.CDIDENTBUTON = TR.CDTIPORECE
        ORDER BY IT.NRPGCONFTELA, IT.NRBUTTON, TR.CDTIPORECE
    ";

    const BUSCA_DADOS_CAIXA = "
        SELECT CDFILIAL, CDCAIXA, NRCONFTELA, CDLOJA, IDTPEMISSAOFOS, IDTPTEF, CDTERTEF
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
    ";

    const BUSCA_DADOS_FILIAL = "
        SELECT F.CDFILIAL, CDCLIENTE, F.NMFILIAL, F.NRORG, F.NRINSJURFILI
          FROM FILIAL F JOIN PARAVEND P
                          ON P.CDFILIAL = F.CDFILIAL
         WHERE F.CDFILIAL = :CDFILIAL
    ";

    const BUSCA_DADOS_PARAVEND = "
        SELECT CDFILIAL, NRATRAPADRAO
          FROM PARAVEND
         WHERE CDFILIAL = :CDFILIAL
    ";

    const BUSCA_DADOS_VENDEDOR = "
        SELECT CDVENDEDOR, NMFANVEN, CDOPERADOR
          FROM VENDEDOR
         WHERE CDVENDEDOR = :CDVENDEDOR
    ";

    const BUSCA_DADOS_OPERADOR = "
        SELECT CDOPERADOR, NMOPERADOR
          FROM OPERADOR
         WHERE CDOPERADOR = :CDOPERADOR
    ";

    const BUSCA_DADOS_LOJA = "
        SELECT NRMESAPADRAO
          FROM LOJA
         WHERE CDFILIAL = :CDFILIAL
           AND CDLOJA = :CDLOJA
    ";

    const SQL_GET_IMP_PRODUTOS = "
        SELECT PA.CDAMBIENTE, IL.NRSEQIMPRLOJA, IL.NMIMPRLOJA
          FROM PRODAMBIENTE PA JOIN IMPRLOJA IL
                                 ON IL.CDFILIAL      = PA.CDFILIAL
                                AND IL.CDLOJA        = PA.CDLOJA
                                AND IL.NRSEQIMPRLOJA = PA.NRSEQIMPRLOJA
         WHERE PA.CDFILIAL   = :CDFILIAL
           AND PA.CDLOJA     = :CDLOJA
           AND PA.CDPRODUTO  = :CDPRODUTO
           AND PA.NRCONFTELA = :NRCONFTELA
    ";

    const SQL_GRUPOS_PROMO_INT = "
        SELECT G.CDGRUPROMOC, P.NMGRUPROMOC, G.QTPRGRUPPROMOC
          FROM GRUPROMOCPROD G JOIN GRUPROMOC P
                                 ON G.CDGRUPROMOC = P.CDGRUPROMOC
         WHERE G.CDPRODPROMOCAO = :CDIDENTBUTON
    ";

    const SQL_GRUPOS_PROMO_INT_FILI = "
        SELECT G.CDGRUPROMOC, P.NMGRUPROMOC, G.QTPRGRUPPROMFIL AS QTPRGRUPPROMOC
          FROM GRUPROMOCPRFIL G JOIN GRUPROMOC P
                                  ON G.CDGRUPROMOC = P.CDGRUPROMOC
         WHERE G.CDPRODPROMOCAO = :CDIDENTBUTON
           AND G.CDFILIAL = :CDFILIAL
    ";

    const ORACLE_PRODUTOS_PROMO_INT = "
        SELECT DP.CDPRODPROMOCAO, PP.CDGRUPROMOC, PP.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.IDIMPPRODUTO,
               NVL(DP.IDPERVALORDES, 'P') AS IDPERVALORDES,  '0' AS IDCTRLMODIFY,
               NVL(DP.VRDESPRODPROMOC, 0) AS VRDESPRODPROMOC,
               NVL(DP.IDAPLICADESCPR, 'T') AS IDAPLICADESCPR,
               'N' AS IDPRODBLOQ, P.DSPRODVENDA, PP.DSENDEIMGPROMO,
               PP.DSAPELIDOMOB, PP.NRORDPROMOPR, P.DSADICPROD
          FROM PRODUTO P JOIN PRODGRUPROMOC PP
                           ON P.CDPRODUTO = PP.CDPRODUTO
                          AND PP.CDGRUPROMOC = :CDGRUPROMOC
                    LEFT JOIN DESGRUPROMOCPR DP
                           ON DP.CDPRODPROMOCAO = :CDIDENTBUTON
                          AND DP.CDGRUPROMOC = PP.CDGRUPROMOC
                          AND DP.CDPRODUTO = PP.CDPRODUTO
         WHERE P.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                              FROM PRODBLOQVND
                                             WHERE CDFILIAL = :CDFILIAL
                                               AND CDLOJA = :CDLOJA
                                               AND (NRDIASEMANABLOQ = 'T'
                                                    OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
        ORDER BY NVL(PP.NRORDPROMOPR , 99999), PP.CDPRODUTO
    ";

    const SQL_PRODUTOS_PROMO_INT = "
        SELECT DP.CDPRODPROMOCAO, PP.CDGRUPROMOC, PP.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.IDIMPPRODUTO,
               ISNULL(DP.IDPERVALORDES, 'P') AS IDPERVALORDES,  '0' AS IDCTRLMODIFY,
               ISNULL(DP.VRDESPRODPROMOC, 0) AS VRDESPRODPROMOC,
               ISNULL(DP.IDAPLICADESCPR, 'T') AS IDAPLICADESCPR,
               'N' AS IDPRODBLOQ, P.DSPRODVENDA, PP.DSENDEIMGPROMO,
               PP.DSAPELIDOMOB, PP.NRORDPROMOPR, P.DSADICPROD
          FROM PRODUTO P JOIN PRODGRUPROMOC PP
                           ON P.CDPRODUTO = PP.CDPRODUTO
                          AND PP.CDGRUPROMOC = :CDGRUPROMOC
                    LEFT JOIN DESGRUPROMOCPR DP
                           ON DP.CDPRODPROMOCAO = :CDIDENTBUTON
                          AND DP.CDGRUPROMOC = PP.CDGRUPROMOC
                          AND DP.CDPRODUTO = PP.CDPRODUTO
             WHERE P.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                              FROM PRODBLOQVND
                                             WHERE CDFILIAL = :CDFILIAL
                                               AND CDLOJA = :CDLOJA
                                               AND (NRDIASEMANABLOQ = 'T'
                                                    OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
        ORDER BY ISNULL(PP.NRORDPROMOPR , 99999), PP.CDPRODUTO
    ";

    const ORACLE_PRODUTOS_PROMO_INT_FILI = "
        SELECT DP.CDPRODPROMOCAO, PP.CDGRUPROMOC, PP.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.IDIMPPRODUTO,
               NVL(DP.IDPERVALDESFIL, 'P') AS IDPERVALORDES, '0' AS IDCTRLMODIFY,
               NVL(DP.VRDESPRPROMOFIL, 0) AS VRDESPRODPROMOC,
               NVL(DP.IDAPLICADESCPRFI, 'T') AS IDAPLICADESCPR,
               'N' AS IDPRODBLOQ, P.DSPRODVENDA, PP.DSENDEIMGPROMO,
               PP.DSAPELIDOMOB, PP.NRORDPROMOPR, P.DSADICPROD
          FROM PRODUTO P JOIN PRODGRUPROMOC PP
                           ON P.CDPRODUTO = PP.CDPRODUTO
                          AND PP.CDGRUPROMOC = :CDGRUPROMOC
                    LEFT JOIN DESGRUPROMPRFIL DP
                           ON DP.CDPRODPROMOCAO = :CDIDENTBUTON
                          AND DP.CDGRUPROMOC = PP.CDGRUPROMOC
                          AND DP.CDPRODUTO = PP.CDPRODUTO
          WHERE P.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                              FROM PRODBLOQVND
                                             WHERE CDFILIAL = :CDFILIAL
                                               AND CDLOJA = :CDLOJA
                                               AND (NRDIASEMANABLOQ = 'T'
                                                    OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
        ORDER BY NVL(PP.NRORDPROMOPR , 99999), PP.CDPRODUTO
    ";

    const SQL_PRODUTOS_PROMO_INT_FILI = "
        SELECT DP.CDPRODPROMOCAO, PP.CDGRUPROMOC, PP.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.IDIMPPRODUTO,
               ISNULL(DP.IDPERVALDESFIL, 'P') AS IDPERVALORDES, '0' AS IDCTRLMODIFY,
               ISNULL(DP.VRDESPRPROMOFIL, 0) AS VRDESPRODPROMOC,
               ISNULL(DP.IDAPLICADESCPRFI, 'T') AS IDAPLICADESCPR,
               'N' AS IDPRODBLOQ, P.DSPRODVENDA, PP.DSENDEIMGPROMO,
               PP.DSAPELIDOMOB, PP.NRORDPROMOPR, P.DSADICPROD
          FROM PRODUTO P JOIN PRODGRUPROMOC PP
                           ON P.CDPRODUTO = PP.CDPRODUTO
                          AND PP.CDGRUPROMOC = :CDGRUPROMOC
                    LEFT JOIN DESGRUPROMPRFIL DP
                           ON DP.CDPRODPROMOCAO = :CDIDENTBUTON
                          AND DP.CDGRUPROMOC = PP.CDGRUPROMOC
                          AND DP.CDPRODUTO = PP.CDPRODUTO
            WHERE P.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                              FROM PRODBLOQVND
                                             WHERE CDFILIAL = :CDFILIAL
                                               AND CDLOJA = :CDLOJA
                                               AND (NRDIASEMANABLOQ = 'T'
                                                    OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
        ORDER BY ISNULL(PP.NRORDPROMOPR , 99999), PP.CDPRODUTO
    ";

    const SQL_BUSCA_AMBIENTES = "
        SELECT IDTPBUTTON, IDTPBUTONAUX, NRBUTTON, NRBUTTONAUX,
               DSBUTTON, CDIDENTBUTON, NRCOLORTEXT, NRCOLORBACK,
               NRPGCONFTELA, NRPGCONFTAUX, DSMESSAGEBUT, DSIMAGEMBUT
          FROM (SELECT AM.IDTPBUTTON, AM.IDTPBUTONAUX, AM.NRBUTTON, AM.NRBUTTONAUX,
                       AM.DSBUTTON, AM.CDIDENTBUTON, AM.NRCOLORTEXT, AM.NRCOLORBACK,
                       AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT, AM.DSIMAGEMBUT
                  FROM ITMENUCONFTE AM
                 WHERE AM.CDFILIAL = :CDFILIAL
                   AND AM.NRCONFTELA = :NRCONFTELA) MESA
         WHERE IDTPBUTTON = '4'
         ORDER BY NRPGCONFTELA, NRBUTTON
    ";

    const SQL_BUSCA_MESAS = "
        SELECT IDTPBUTTON, IDTPBUTONAUX, NRBUTTON, NRBUTTONAUX,
               DSBUTTON, CDIDENTBUTON, NRCOLORTEXT, NRCOLORBACK,
               NRPGCONFTELA, NRPGCONFTAUX, DSMESSAGEBUT, DSIMAGEMBUT
          FROM (SELECT AM.IDTPBUTTON, AM.IDTPBUTONAUX, AM.NRBUTTON, AM.NRBUTTONAUX,
                       AM.DSBUTTON, AM.CDIDENTBUTON, AM.NRCOLORTEXT, AM.NRCOLORBACK,
                       AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT, AM.DSIMAGEMBUT
                  FROM ITMENUCONFTE AM
                 WHERE AM.CDFILIAL  = :CDFILIAL
                   AND AM.NRCONFTELA = :NRCONFTELA) MESA
         WHERE IDTPBUTTON = '3'
           AND IDTPBUTONAUX = '4'
           AND CDIDENTBUTON <> ISNULL(:NRMESAPADRAO, 'XXXX')
        ORDER BY NRPGCONFTELA, NRBUTTON
    ";

    const BUSCA_OBSERVACOES_SQL = "
        SELECT DISTINCT PR.CDOCORR, OC.DSOCORR, '#' + OC.NRCORSINAL AS NRCORSINAL, GR.CDGRPOCOR,
               OC.IDCONTROLAOBS, OC.DSENDEIMGOCORR, OC.CDPRODUTO, OC.IDCONTROLAOBS,
               OC.DSAPELIDOOBS, OC.IDEXIOBSAPPCONS
          FROM GRUPOCOR GR JOIN PRODOCORRE PR
                             ON GR.CDGRPOCOR = PR.CDGRPOCOR
                           JOIN LOJA LJ
                             ON GR.CDGRPOCOR = LJ.CDGRPOCORPED
                           JOIN OCORRENCIA OC
                             ON PR.CDGRPOCOR = OC.CDGRPOCOR
                            AND PR.CDOCORR = OC.CDOCORR
         WHERE LJ.CDFILIAL = :CDFILIAL
           AND LJ.CDLOJA = :CDLOJA
    ";

    const BUSCA_OBSERVACOES_ORACLE = "
        SELECT DISTINCT PR.CDOCORR, OC.DSOCORR, '#' || OC.NRCORSINAL AS NRCORSINAL, GR.CDGRPOCOR,
               OC.IDCONTROLAOBS, OC.DSENDEIMGOCORR, OC.CDPRODUTO, OC.IDCONTROLAOBS,
               OC.DSAPELIDOOBS, OC.IDEXIOBSAPPCONS
          FROM GRUPOCOR GR JOIN PRODOCORRE PR
                             ON GR.CDGRPOCOR = PR.CDGRPOCOR
                           JOIN LOJA LJ
                             ON GR.CDGRPOCOR = LJ.CDGRPOCORPED
                           JOIN OCORRENCIA OC
                             ON PR.CDGRPOCOR = OC.CDGRPOCOR
                            AND PR.CDOCORR = OC.CDOCORR
         WHERE LJ.CDFILIAL = :CDFILIAL
           AND LJ.CDLOJA = :CDLOJA
           AND NVL(OC.CDPRODUTO,'NOTHING') NOT IN (SELECT CDPRODUTO
                                                     FROM PRODBLOQVND
                                                    WHERE CDFILIAL = :CDFILIAL
                                                   AND CDLOJA = :CDLOJA
                                                   AND (NRDIASEMANABLOQ = 'T'
                                                        OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
    ";


    const BUSCA_OBSERVACOES_PRODUTO = "
        SELECT PR.CDPRODUTO, PR.CDOCORR, OC.DSOCORR, OC.CDGRPOCOR,
               OC.IDCONTROLAOBS
          FROM GRUPOCOR GR, PRODOCORRE PR, LOJA LJ, OCORRENCIA OC
         WHERE LJ.CDFILIAL  = :CDFILIAL
           AND LJ.CDLOJA    = :CDLOJA
           AND GR.CDGRPOCOR = LJ.CDGRPOCORPED
           AND PR.CDGRPOCOR = GR.CDGRPOCOR
           AND GR.CDGRPOCOR = OC.CDGRPOCOR
           AND PR.CDOCORR   = OC.CDOCORR
           AND PR.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                      FROM PRODBLOQVND
                                     WHERE CDFILIAL = :CDFILIAL
                                       AND CDLOJA = :CDLOJA
                                       AND (NRDIASEMANABLOQ = 'T'
                                            OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
    ";

    const BUSCA_GRUPO_PRODUTOS = "
        SELECT IDTPBUTTON, IDTPBUTONAUX, NRBUTTON, NRBUTTONAUX,
               DSBUTTON, CDIDENTBUTON, NRCOLORTEXT, NRCOLORBACK,
               NRPGCONFTELA, NRPGCONFTAUX, DSMESSAGEBUT, DSIMAGEMBUT,
               DSENDEIMG
          FROM (SELECT AM.IDTPBUTTON, AM.IDTPBUTONAUX, AM.NRBUTTON, AM.NRBUTTONAUX,
                       AM.DSBUTTON, AM.CDIDENTBUTON, AM.NRCOLORTEXT, AM.NRCOLORBACK,
                       AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT, AM.DSIMAGEMBUT,
                       AM.DSENDEIMG
                  FROM ITMENUCONFTE AM
                 WHERE AM.CDFILIAL   = :CDFILIAL
                   AND AM.NRCONFTELA = :NRCONFTELA) MESA
         WHERE IDTPBUTTON = '2'
         ORDER BY NRPGCONFTELA, NRBUTTON
    ";

    const GET_COMBO_PRODUCTS = "
        SELECT AM.IDTPBUTTON, AM.IDTPBUTONAUX, AM.NRBUTTON, AM.NRBUTTONAUX,
               AM.DSBUTTON, AM.CDIDENTBUTON, AM.NRCOLORTEXT, AM.NRCOLORBACK,
               AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT, AM.DSIMAGEMBUT,
               'N' AS IDSOLLOCIMP, 'N' AS IDPESAPROD, NULL AS CDARVPROD, NULL AS CDPRODINTE,
               NULL AS CDBARPRODUTO, NULL AS IDTIPOPROD, '0' AS IDTIPOCOMPPROD, '1' AS IDIMPPRODUTO,
               NULL AS DTINIVGPROMOC, NULL AS DTFINVGPROMOC, NULL AS CDPRODUTO, 'N' AS IDCONTROLAREFIL,
               AM.CDFILIAL, AM.NRCONFTELA, AM.DSENDEIMG, AM.IDTPBUTONAUX2,
               AM.NRBUTTONAUX2, AM.NRPGCONFTAUX2, 'N' AS IDPRODBLOQ,
               AM.NRLIMPRODCOM, NULL AS DSPRODVENDA, L.IDTIPCOBRA
          FROM ITMENUCONFTE AM
          JOIN LOJA L ON L.CDFILIAL = AM.CDFILIAL
         WHERE AM.CDFILIAL = :CDFILIAL
           AND L.CDLOJA = :CDLOJA
           AND AM.NRCONFTELA = :NRCONFTELA
           AND AM.IDTPBUTTON = 'B'
    ";

    const BUSCA_SUBGRUPO_PRODUTOS = "
        SELECT IDTPBUTTON, IDTPBUTONAUX, NRBUTTON, NRBUTTONAUX,
               DSBUTTON, CDIDENTBUTON, NRCOLORTEXT, NRCOLORBACK,
               NRPGCONFTELA, NRPGCONFTAUX, DSMESSAGEBUT, DSIMAGEMBUT,
               DSENDEIMG
          FROM ITMENUCONFTE
         WHERE IDTPBUTTON = 'I'
           AND CDFILIAL = :CDFILIAL
           AND NRCONFTELA = :NRCONFTELA
    ";

    const BUSCA_PRODUTOS_ORACLE = "
        SELECT IDTPBUTTON, IDTPBUTONAUX, NRBUTTON,
               NRBUTTONAUX, DSBUTTON, CDIDENTBUTON,
               NRCOLORTEXT, NRCOLORBACK, NRPGCONFTELA,
               NRPGCONFTAUX, DSMESSAGEBUT, DSIMAGEMBUT,
               IDSOLLOCIMP, IDPESAPROD, CDARVPROD,
               CDPRODINTE, CDBARPRODUTO, IDTIPOPROD,
               IDTIPOCOMPPROD, IDIMPPRODUTO, DTINIVGPROMOC, DTFINVGPROMOC,
               CDPRODUTO, IDCONTROLAREFIL, CDFILIAL, NRCONFTELA, DSENDEIMG,
               IDTPBUTONAUX2, NRBUTTONAUX2, NRPGCONFTAUX2, DSPRODVENDA, 'N' AS IDPRODBLOQ,
               DSADICPROD
          FROM (SELECT AM.IDTPBUTTON, AM.IDTPBUTONAUX, AM.NRBUTTON, AM.NRBUTTONAUX,
                       AM.DSBUTTON, AM.CDIDENTBUTON, AM.NRCOLORTEXT, AM.NRCOLORBACK,
                       AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT,
                       AM.DSIMAGEMBUT, PR.IDSOLLOCIMP, PR.IDPESAPROD, PR.CDARVPROD,
                       PR.CDPRODINTE, PR.CDBARPRODUTO, PR.IDTIPOPROD, PR.IDTIPOCOMPPROD,
                       PR.IDIMPPRODUTO, PR.DTINIVGPROMOC, PR.DTFINVGPROMOC, PR.CDPRODUTO,
                       PR.IDCONTROLAREFIL, AM.CDFILIAL, AM.NRCONFTELA, NVL(AM.DSENDEIMG, PR.DSENDEIMG) AS DSENDEIMG, AM.IDTPBUTONAUX2,
                       AM.NRBUTTONAUX2, AM.NRPGCONFTAUX2, PR.DSPRODVENDA, PR.DSADICPROD

                  FROM ITMENUCONFTE AM

                  JOIN PRODUTO PR
                    ON AM.CDIDENTBUTON = PR.CDPRODUTO
                  JOIN CONFTELA CF
                    ON AM.CDFILIAL = CF.CDFILIAL
                   AND AM.NRCONFTELA = CF.NRCONFTELA

                 WHERE AM.CDFILIAL = :CDFILIAL
                   AND AM.NRCONFTELA = :NRCONFTELA
                   AND PR.CDPRODUTO NOT IN (SELECT CDPRODUTO

                                              FROM PRODBLOQVND

                                             WHERE CDFILIAL = :CDFILIAL
                                               AND CDLOJA = :CDLOJA
                                               AND (NRDIASEMANABLOQ = 'T'
                                                    OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ)) ) MESA
          WHERE IDTPBUTTON = '1'
          AND IDTPBUTONAUX IN ('2', 'I')
          ORDER BY NRPGCONFTAUX, NRBUTTONAUX, NRPGCONFTELA, NRBUTTON
    ";

    const BUSCA_PRODUTOS_SQL = "
        SELECT IDTPBUTTON, IDTPBUTONAUX, NRBUTTON,
               NRBUTTONAUX, DSBUTTON, CDIDENTBUTON,
               NRCOLORTEXT, NRCOLORBACK, NRPGCONFTELA,
               NRPGCONFTAUX, DSMESSAGEBUT, DSIMAGEMBUT,
               IDSOLLOCIMP, IDPESAPROD, CDARVPROD,
               CDPRODINTE, CDBARPRODUTO, IDTIPOPROD,
               IDTIPOCOMPPROD, IDIMPPRODUTO, DTINIVGPROMOC, DTFINVGPROMOC,
               CDPRODUTO, IDCONTROLAREFIL, CDFILIAL, NRCONFTELA, DSENDEIMG,
               IDTPBUTONAUX2, NRBUTTONAUX2, NRPGCONFTAUX2, DSPRODVENDA, 'N' AS IDPRODBLOQ,
               DSADICPROD
          FROM (SELECT AM.IDTPBUTTON, AM.IDTPBUTONAUX, AM.NRBUTTON, AM.NRBUTTONAUX,
                       AM.DSBUTTON, AM.CDIDENTBUTON, AM.NRCOLORTEXT, AM.NRCOLORBACK,
                       AM.NRPGCONFTELA, AM.NRPGCONFTAUX, AM.DSMESSAGEBUT,
                       AM.DSIMAGEMBUT, PR.IDSOLLOCIMP, PR.IDPESAPROD, PR.CDARVPROD,
                       PR.CDPRODINTE, PR.CDBARPRODUTO, PR.IDTIPOPROD, PR.IDTIPOCOMPPROD,
                       PR.IDIMPPRODUTO, PR.DTINIVGPROMOC, PR.DTFINVGPROMOC, PR.CDPRODUTO,
                       PR.IDCONTROLAREFIL, AM.CDFILIAL, AM.NRCONFTELA, ISNULL(AM.DSENDEIMG, PR.DSENDEIMG) AS DSENDEIMG, AM.IDTPBUTONAUX2,
                       AM.NRBUTTONAUX2, AM.NRPGCONFTAUX2, PR.DSPRODVENDA, PR.DSADICPROD

                  FROM ITMENUCONFTE AM

                  JOIN PRODUTO PR
                    ON AM.CDIDENTBUTON = PR.CDPRODUTO
                  JOIN CONFTELA CF
                    ON AM.CDFILIAL = CF.CDFILIAL
                   AND AM.NRCONFTELA = CF.NRCONFTELA

                 WHERE AM.CDFILIAL = :CDFILIAL
                   AND AM.NRCONFTELA = :NRCONFTELA
                   AND PR.CDPRODUTO NOT IN (SELECT CDPRODUTO

                                              FROM PRODBLOQVND

                                             WHERE CDFILIAL = :CDFILIAL
                                               AND CDLOJA = :CDLOJA
                                               AND (NRDIASEMANABLOQ = 'T'
                                                    OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ)) ) MESA
          WHERE IDTPBUTTON = '1'
          AND IDTPBUTONAUX IN ('2', 'I')
          ORDER BY NRPGCONFTAUX, NRBUTTONAUX, NRPGCONFTELA, NRBUTTON
    ";

    const BUSCA_PRODUTOS_COMBO_ORACLE = "
        SELECT P.CDPRODUTO AS CDPRODUTO, P.IDIMPPRODUTO, 'T' AS IDAPLICADESCPR, 'P' AS IDPERVALORDES,
               NVL(I.DSBUTTON, P.NMPRODUTO) AS NMPRODUTO, '.000' AS VRDESPRODPROMOC,
               'N' AS IDPRODBLOQ, 'COMBO' AS NMGRUPROMOC, NVL(I.CDIDENTBUTON, P.CDPRODUTO) AS CDIDENTBUTON,
               NVL(I.DSBUTTON, P.NMPRODUTO) AS DSOCORR, P.CDARVPROD AS CDOCORR, P.CDARVPROD,
               P.IDTIPOPROD, P.IDTIPOCOMPPROD, NVL(I.DSBUTTON, P.NMPRODUTO) AS NOME, P.DSPRODVENDA AS DESCRIPTION
          FROM PRODUTO P LEFT JOIN ITMENUCONFTE I
                                ON P.CDPRODUTO = I.CDIDENTBUTON
                               AND I.CDFILIAL = :CDFILIAL
                               AND I.NRCONFTELA = :NRCONFTELA
         WHERE P.IDTIPOPROD = :IDTIPOPROD
        AND P.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                  FROM PRODBLOQVND
                                 WHERE CDFILIAL = :CDFILIAL
                                   AND CDLOJA = :CDLOJA
                                   AND (NRDIASEMANABLOQ = 'T'
                                        OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
    ";

    const BUSCA_PRODUTOS_COMBO_SQL = "
        SELECT P.CDPRODUTO AS CDPRODUTO, P.IDIMPPRODUTO, 'T' AS IDAPLICADESCPR, 'P' AS IDPERVALORDES,
               ISNULL(I.DSBUTTON, P.NMPRODUTO) AS NMPRODUTO, '.000' AS VRDESPRODPROMOC,
               'N' AS IDPRODBLOQ, 'COMBO' AS NMGRUPROMOC, ISNULL(I.CDIDENTBUTON, P.CDPRODUTO) AS CDIDENTBUTON,
               ISNULL(I.DSBUTTON, P.NMPRODUTO) AS DSOCORR, P.CDARVPROD AS CDOCORR, P.CDARVPROD,
               P.IDTIPOPROD, P.IDTIPOCOMPPROD, ISNULL(I.DSBUTTON, P.NMPRODUTO) AS NOME, P.DSPRODVENDA AS DESCRIPTION
          FROM PRODUTO P LEFT JOIN ITMENUCONFTE I
                                ON P.CDPRODUTO = I.CDIDENTBUTON
                               AND I.CDFILIAL = :CDFILIAL
                               AND I.NRCONFTELA = :NRCONFTELA
         WHERE P.IDTIPOPROD = :IDTIPOPROD
        AND P.CDPRODUTO NOT IN (SELECT CDPRODUTO
                                  FROM PRODBLOQVND
                                 WHERE CDFILIAL = :CDFILIAL
                                   AND CDLOJA = :CDLOJA
                                   AND (NRDIASEMANABLOQ = 'T'
                                        OR  NRDIASEMANABLOQ = :NRDIASEMANABLOQ))
    ";

    const SQL_PRECOS = "
        SELECT CDPRODUTO, VRPRECITEM
          FROM ITEMPRECO
         WHERE CDFILIAL = :CDFILIAL
           AND CDTABEPREC = :CDTABEPREC
           AND DTINIVGPREC = CONVERT(DATETIME, :DTINIVGPREC, 120)
    ";

    const ORACLE_PRECOS = "
        SELECT CDPRODUTO, VRPRECITEM
          FROM ITEMPRECO
         WHERE CDFILIAL = :CDFILIAL
           AND CDTABEPREC = :CDTABEPREC
           AND DTINIVGPREC = TRUNC(TO_DATE(:DTINIVGPREC, 'DD-MM-YYYY HH24:MI:SS'))
    ";

    const ORACLE_PRECOS_DIA = "
        SELECT VRPRECODIA, IDPERVALORPR, IDDESCACREPR, IDVISUACUPOM, CDTIPOCONSPD, CDPRODUTO
          FROM ITEMPRECODIA
         WHERE CDFILIAL       = :CDFILIAL
           AND CDTABEPREC     = :CDTABEPREC
           AND DTINIVGPREC    = TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH:MI:SS')
           AND ((NRDIASEMANPR = :NRDIASEMANPR) OR (NRDIASEMANPR = 'T'))
           AND TO_CHAR(SYSDATE,'HH24')||'00' BETWEEN HRINIPRECDIA AND HRFINPRECDIA
           AND CDTIPOCONSPD = :CDTIPOCONSPD
    ";

    const SQL_PRECOS_DIA = "
        SELECT VRPRECODIA, IDPERVALORPR, IDDESCACREPR, IDVISUACUPOM, CDTIPOCONSPD, CDPRODUTO
          FROM ITEMPRECODIA
         WHERE CDFILIAL       = :CDFILIAL
           AND CDTABEPREC     = :CDTABEPREC
           AND DTINIVGPREC    = CONVERT(DATETIME, :DTINIVGPREC, 120)
           AND ((NRDIASEMANPR = :NRDIASEMANPR) OR (NRDIASEMANPR = 'T'))
           AND :HORA BETWEEN HRINIPRECDIA AND HRFINPRECDIA
           AND (CONVERT(VARCHAR, GETDATE(), 103) BETWEEN DTINIVALPREC AND DTFINVALPREC)
           AND CDTIPOCONSPD = :CDTIPOCONSPD
    ";

    const SQL_PREC_FILIAL = "
        SELECT CDTABEPREC
          FROM PARAVEND
         WHERE CDFILIAL = :CDFILIAL
    ";

    const SQL_BUSCA_VIGENCIA = "
        SELECT CONVERT(VARCHAR,DTINIVGPREC,21) AS DTINIVGPREC
          FROM TABEVEND
         WHERE CDFILIAL = :CDFILIAL
           AND CDTABEPREC = :CDTABEPREC
           AND (CONVERT(VARCHAR, GETDATE(), 103) BETWEEN DTINIVGPREC AND DTFINVGPREC)
    ";

    const SQL_PREC_CLIENTE = "
        SELECT CDFILTABPREC, CDTABEPREC
          FROM CLIENTE
         WHERE CDCLIENTE = :CDCLIENTE
    ";

    const SQL_PREC_LOJA = "
        SELECT CDTABEPREC
          FROM LOJA
         WHERE CDFILIAL = :CDFILIAL
           AND CDLOJA = :CDLOJA
    ";

    const SQL_VAL_TABE = "
        SELECT DTINIVGPREC
          FROM TABEVEND
         WHERE CDFILIAL = :CDFILIAL
           AND CDTABEPREC = :CDTABEPREC
           AND (:DATE BETWEEN DTINIVGPREC AND DTFINVGPREC)
    ";
}

class Bench {
    private $start;
    private $end;

    public function __construct() {
        $this->start = microtime(true);
    }

    public function end() {
        $this->end = microtime(true);
        return ($this->end - $this->start);
    }
}