describe("EventAggregator", function () {
	var controller;
	var repository;
	var repositoryRetry;
	var $rootScope;
	var $httpBackend;
	var EventAggregator;
	beforeEach(function () {
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.serviceUrl = "../backend/service/index.php";
		module("ZeedhiApp");
		inject(function (eventAggregator) {
			EventAggregator = eventAggregator;
		});
	});

	it("Should set the requestConfig property to the onRequestStart event data", function () {
		var eventDataToCheck;
		EventAggregator.subscribe("EventOnRequestStart", function (eventData) {
			eventDataToCheck = eventData;
		});
		EventAggregator.onRequestStart({
			data: { timeout: 1000 },
			index: 1,
			requestConfig: { prop: "val" }
		});

		expect(eventDataToCheck).toBeDefined();
		expect(eventDataToCheck.requestConfig).toEqual({ prop: "val" });
	});

	it("Should set the requestConfig property to the onRequestMetaDataStart event data", function () {
		var eventDataToCheck;
		EventAggregator.subscribe("EventOnRequestMetaDataStart", function (eventData) {
			eventDataToCheck = eventData;
		});
		EventAggregator.onRequestMetaDataStart({
			data: { timeout: 1000 },
			requestConfig: { prop: "val" }
		});

		expect(eventDataToCheck).toBeDefined();
		expect(eventDataToCheck.requestConfig).toEqual({ prop: "val" });
	});

	describe(" Repository download", function () {
		beforeEach(inject(function (_$controller_, _$rootScope_, _$httpBackend_) {
			$rootScope = _$rootScope_;
			$httpBackend = _$httpBackend_;
			// The injector unwraps the underscores (_) from around the parameter names when matching
			$controller = _$controller_;
			var $scope = $rootScope;
			controller = $controller("ZeedhiMain", {
				$scope: $scope
			});
			$httpBackend.when("GET", "/json_test/test/json/acl.json").respond({}, {});
			$httpBackend.when("GET", "/json_test/test/json/words.json").respond({}, {});
			$httpBackend.when("GET", "/json_test/test/json/data.json").respond({}, {});
			$httpBackend.when("GET", "/json_test/test/json/datasources/repository.json").respond({}, {});
			$httpBackend.when("GET", "/json_test/test/json/containers.json").respond(ZhHelper.projectTest(), {});
			$httpBackend.when("GET", "/json_test/test/json/containers/login.json").respond(
				ZhHelper.emptyContainer, {});
			//inject();
			$rootScope.$apply();
		}));

		it("EventAggregator do not be undefined", function () {
			expect(EventAggregator).not.toEqual(undefined);
		});

		it("EventAggregator should have many methods", function () {
			expect(EventAggregator.onRequestStart).not.toEqual(undefined);
			expect(EventAggregator.onRequestEnd).not.toEqual(undefined);
			expect(EventAggregator.onRequestRetry).not.toEqual(undefined);
			expect(EventAggregator.onRequestSuccess).not.toEqual(undefined);
			expect(EventAggregator.onRequestError).not.toEqual(undefined);
		});

		it("EventAggregator should subscribe and publish an event correctly", function (done) {
			var eventData = {};
			var fn = function (e) {
				expect(e).toEqual(eventData);
				done();
			};
			var id = EventAggregator.subscribe("onOk", fn);
			EventAggregator.publish("onOk", eventData);
		});

		it("EventAggregator should unSubscribe", function () {
			var eventMock = {
				fn: function (eventData) {
				}
			};
			spyOn(eventMock, "fn").and.callThrough();
			var id = EventAggregator.subscribe("onOk", eventMock.fn);
			EventAggregator.unSubscribe("onOk", id);
			EventAggregator.publish("onOk", {});
			expect(eventMock.fn).not.toHaveBeenCalled();
		});
	});
});