function CustomizationPreferences(ScreenService, templateManager, zhCustomizationsService, visualizationsWidgetsCreator, zhLayoutPreferences, zhFilterPreferences, eventEngine, eventAggregator) {

	var self = this;
	var editingWidget;

	eventAggregator.subscribe("OnListStartUp", getAndApplyDefaultPreference);

	function getAndApplyDefaultPreference(eventData) {
		if (eventData.widget) {
			eventData.widget.customizationsApplied = eventData.widget.customizationsApplied || {};
			eventData.widget.customizationsApplied.view = "default";
		}
	}

	this.startViewSettings = function startViewSettings(editingWidgetParameter) {
		editingWidget = editingWidgetParameter;
		var popUpWidget = visualizationsWidgetsCreator.createPopupVisualizations(editingWidget, functionsVisualizationsListWidget);
		ScreenService.openPopup(popUpWidget);
	};

	this.restoreDefault = function restoreDefault(editingWidget) {
		restoreDefaultContainer(editingWidget);
	};

	var visualizationsListOnEnter = function visualizationsListOnEnter(widget) {
		return zhCustomizationsService.getWidgetCustomizations(editingWidget, "view").then(function (views) {
			views.forEach(function (view) {
				view.viewRepositoryLabel = zhCustomizationsService.getLabelRepositoryType(view.customizationRepository);
			});
			widget.dataSource.data = views;
		});
	};

	function applyVisualization(view, gridWidget) {
		gridWidget = gridWidget || editingWidget;
		var layoutId = view.preferenceId;
		var filterId = view.filterId;
		if(!gridWidget.customizationsApplied){
			gridWidget.customizationsApplied = {};
		}
		gridWidget.customizationsApplied.view = view._id;
		zhCustomizationsService.getWidgetCustomizationsByRepository(gridWidget, "layout", view.customizationRepository, layoutId).then(function(preference){
			preference = preference.shift();
			zhLayoutPreferences.applyLayout(preference, gridWidget);
			zhCustomizationsService.getWidgetCustomizationsByRepository(gridWidget, "filter", view.customizationRepository, filterId).then(function(filter){
				filter = filter.shift();
				zhFilterPreferences.applyFilter(filter, gridWidget);
			});
		});
	}

	function applyVisualizationEventCallback(eventData){
		var view = eventData.e.customization;
		var editingWidget = eventData.e.editingWidget;
		if(!view.isDefaultOption){
			applyVisualization(view, editingWidget);
		} else {
			restoreDefaultContainer(editingWidget);
		}
	}

	eventEngine.bindEvent(zhCustomizationsService, "VisualizationPreferenceOnClick", applyVisualizationEventCallback);

	function restoreDefaultContainer(editingWidget) {
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		editingWidget.customizationsApplied.view = "default";

		zhLayoutPreferences.restoreDefaultLayout(editingWidget);
		zhFilterPreferences.restoreDefaultFilter(editingWidget);
	}

	function restoreDefaultContainerActionClick() {
		restoreDefaultContainer(editingWidget);
		ScreenService.closePopup();
	}

	var openFormEditVisualization = function openFormEditVisualization(widget, row) {
		getVisualizationsForm("edit").then(function (formWidget) {
			formWidget.getField('label').value(row.label);
			formWidget.getField('customizationRepository').value(row.customizationRepository);
			formWidget.getField('customizationRepository').readOnly = true;
			formWidget.getField('viewPreference').value(row.preferenceId);
			formWidget.getField('viewFilter').value(row.filterId);
			formWidget.userId = row.userId;
			formWidget.viewId = row._id;
			ScreenService.openPopup(formWidget);
		});
	};

	var openFormNewVisualization = function openFormNewVisualization() {
		getVisualizationsForm("new").then(function (formWidget) {
			formWidget.newRow();
			ScreenService.openPopup(formWidget);
		});
	};

	function getVisualizationsForm(formType, parentWidget) {
		var functions;
		if (formType == "edit") {
			functions = functionsEditVisualization;
		} else if (formType == "new") {
			functions = functionsNewVisualization;
		}
		var formWidget = visualizationsWidgetsCreator.createPopupViewCreator(editingWidget, functions, parentWidget);
		var preferencesField = formWidget.getField('viewPreference');
		var filtersField = formWidget.getField('viewFilter');
		return zhCustomizationsService.getWidgetCustomizations(editingWidget, "layout").then(function (preferences) {
			preferencesField.dataSource.data = preferences;
			return zhCustomizationsService.getWidgetCustomizations(editingWidget, "filter").then(function (filters) {
				filtersField.dataSource.data = filters;
				formWidget.getField('customizationRepository').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
				return formWidget;
			});
		});
	}

	var onSelectRepository = function (widget, field) {
		var preferencesField = widget.getField('viewPreference');
		var filtersField = widget.getField('viewFilter');
		preferencesField.clearValue();
		filtersField.clearValue();
		preferencesField.readOnly = !field.value();
		filtersField.readOnly = !field.value();
		if (field.value()) {
			zhCustomizationsService.getWidgetCustomizationsByRepository(editingWidget, "layout", field.value()).then(function (preferences) {
				preferencesField.dataSource.data = preferences;
				zhCustomizationsService.getWidgetCustomizationsByRepository(editingWidget, "filter", field.value()).then(function (filters) {
					filtersField.dataSource.data = filters;
				});
			});
		}
	};

	var createNewView = function (widget) {
		if (widget.isValid()) {
			var viewRegisterData = createDatasetView(widget);
			zhCustomizationsService.saveCustomization(viewRegisterData, 'view').then(function () {
				visualizationsListOnEnter(widget).then(function () {
					applyVisualization(viewRegisterData, editingWidget);
					ScreenService.closePopup(true);
					templateManager.updateTemplate();
				});
			});
		}
	};

	var createDatasetView = function (widget, id) {
		var viewDataObject = {
			"label": widget.currentRow.label,
			"filterId": widget.currentRow.viewFilter,
			"preferenceId": widget.currentRow.viewPreference,
			"customizationRepository": widget.currentRow.customizationRepository,
			"containerId": editingWidget.container.id,
			"widgetId": editingWidget.id,
			"customizationType": "view"
		};
		if (widget.userId) {
			viewDataObject.userId = widget.userId;
		}

		if (!id) {
			viewDataObject.unsync_id = new Date().getTime();
		} else {
			viewDataObject._id = id;
		}
		return viewDataObject;
	};

	var deleteVisualizations = function (widget) {
		if (widget.getCheckedRows().length > 0) {
			if (widget.getCheckedRows().some(function(row) {
				return editingWidget.customizationsApplied.view == row._id;
			})) {
				restoreDefaultContainer(editingWidget);
			}

			zhCustomizationsService.removeCustomizations(widget.getCheckedRows()).then(function () {
				visualizationsListOnEnter(widget).then(function () {
					templateManager.updateTemplate();
				});
			});
		}
	};

	var editView = function (widget, row) {
		var viewId = widget.viewId;
		if (widget.isValid()) {
			var viewRegisterData = createDatasetView(widget, viewId);
			zhCustomizationsService.saveCustomization(viewRegisterData, 'view').then(function () {
				visualizationsListOnEnter(widget).then(function () {
					applyVisualization(viewRegisterData, editingWidget);
					ScreenService.closePopup(true);
					templateManager.updateTemplate();
				});
			});
		}
	};

	var openPopupSelectDestinyRepository = function (parentWidget) {
		var changePreferenceTypeWidget = visualizationsWidgetsCreator.createChangePreferenceTypeWidget(editingWidget, functionsTransferCustomizationRepository, parentWidget);
		changePreferenceTypeWidget.newRow();
		var allowedPreferences = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		changePreferenceTypeWidget.getField("preferenceType").dataSource.data = allowedPreferences;
		ScreenService.openPopup(changePreferenceTypeWidget);
	};

	var saveTransferLayoutClick = function (widgetChangePreferenceType) {
		var tranferConfirmationCallback = function () {
			var preferencesListWidget = widgetChangePreferenceType.parentWidget;
			var toChangePreferences = preferencesListWidget.getCheckedRows();
			var newLayoutType = widgetChangePreferenceType.getField("preferenceType").value();
			zhCustomizationsService.transferCustomizationsRepository(editingWidget, toChangePreferences, newLayoutType).then(function () {
				visualizationsListOnEnter(preferencesListWidget).then(function () {
					ScreenService.closePopup();
				});
			});
		};
		ScreenService.confirmMessage(ScreenService.i18n("If you transfer this view all connected customizations will be transfered too. Are you sure?"), "CONFIRMATION", tranferConfirmationCallback);
	};

	var functionsVisualizationsListWidget = {
		"deleteVisualizations": deleteVisualizations,
		"applyVisualization": applyVisualization,
		"openFormNewVisualization": openFormNewVisualization,
		"openFormEditVisualization": openFormEditVisualization,
		"restoreDefaultContainerActionClick": restoreDefaultContainerActionClick,
		"visualizationsListOnEnter": visualizationsListOnEnter,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository
	};

	var functionsNewVisualization = {
		"save": createNewView,
		"onSelectRepository": onSelectRepository
	};

	var functionsEditVisualization = {
		"save": editView,
		"onSelectRepository": onSelectRepository
	};

	var functionsTransferCustomizationRepository = {
		"saveTransferLayoutClick": saveTransferLayoutClick
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register("zhCustomizationPreferences", CustomizationPreferences);
});