"use strict";

// FILE: src/service/MenuService.js
function MenuService(ScreenService, Query, ZhUtil, RepositoryFactory, eventEngine, templateManager) {
	var _this2 = this;

	var self = this;
	var MENU_ITEM_ID = "CDPRODUTO";
	var QTD_ITEM = "QUANTIDADE";

	this.selectedProduct = null;
	this.cart = [];

	this.addToCart = function (menuWidget) {
		var _this = this;

		var promise = new Promise(function (resolve, reject) {
			var product = self.selectedProduct;
			if (product) {
				if (product.IDTIPOCOMPPROD == '3' || product.IDTIPOCOMPPROD == 'C') {
					// Adding combo product
					if (_isComboCartFilled(product.comboCart)) {
						var existentComboData = _getExistentComboInCart(product, _this.cart);
						if (existentComboData.exist) {
							var alreadyExistentProduct = _this.cart[existentComboData.productIndex];
							alreadyExistentProduct[QTD_ITEM] += product[QTD_ITEM];
						} else {
							_this.cart.push(product);
						}
						self.updateCartHint(templateManager.container.getWidget('menu'));
						resolve(product);
						ScreenService.closePopup(true);
					} else {
						reject('Preencha todos os itens para prosseguir.');
					}
				} else if (product.parentProduct) {
					// Adding product from combo
					var parentComboProduct = menuWidget.widgets.first().product;
					var currentComboCart = parentComboProduct.comboCart[product.parentComboItem];
					if (self.getTotalCurrentComboProducts(currentComboCart.products) + product.QUANTIDADE <= currentComboCart.QTPRGRUPPROMOC) {
						currentComboCart.products.push(product);
						resolve(product);
					} else {
						reject('Quantidade de produtos para o grupo excedida.');
					}
					// Resets selected product to combo again
					self.selectedProduct = parentComboProduct;
					ScreenService.closePopup();

					// Checks if cart is filled to add combo automatically and improve ux
					_performAutomaticSelect(currentComboCart, menuWidget);
				} else {
					// Adding normal product
					var cartItemCode = product[MENU_ITEM_ID];
					var existentProductData = _getExistentProductInCart(cartItemCode, _this.cart);
					if (existentProductData.exist) {
						var _alreadyExistentProduct = _this.cart[existentProductData.productIndex];
						_alreadyExistentProduct[QTD_ITEM] += product[QTD_ITEM];
					} else {
						_this.cart.push(product);
					}

					self.updateCartHint(templateManager.container.getWidget('menu'));
					resolve(product);
					ScreenService.closePopup();
				}
			} else {
				reject('Erro ao adicionar o item ao carrinho. Confira se os parametros utilizados no template estão de acordo com a documentação.');
			}
		});
		return promise;
	};

	var _performAutomaticSelect = function _performAutomaticSelect(comboCart, menuWidget) {
		var totalComboLength = comboCart.QTPRGRUPPROMOC;
		var comboCartLength = _this2.getTotalCurrentComboProducts(comboCart.products);

		if (parseFloat(totalComboLength) === comboCartLength) {
			_this2.addToCart(menuWidget);
		}
	};

	this.getTotalPrice = function () {
		var totalPrice = 0;
		this.cart.forEach(function (cartItem) {
			if (cartItem.IDTIPOCOMPPROD == '3' || cartItem.IDTIPOCOMPPROD == 'C') {
				totalPrice += self.calculateComboPrice(cartItem);
			} else {
				totalPrice += cartItem.QUANTIDADE * parseFloat(cartItem.VRPRECITEM);
			}
		});
		return totalPrice;
	};

	this.calculateComboPrice = function (combo) {
		var totalComboPrice = 0;

		for (var i in combo.comboCart) {
			combo.comboCart[i].products.forEach(function (comboItem, index) {
				totalComboPrice += comboItem.QUANTIDADE * parseFloat(comboItem.VRPRECITEM);
			});
		}

		return totalComboPrice;
	};

	this.getCart = function () {
		return this.cart;
	};

	this.updateCartHint = function (menuWidget) {
		var cartAction = menuWidget.getAction('cart');
		if (cartAction) {
			cartAction.hint = this.cart.length;
		}
	};

	this.clearCart = function () {
		this.cart = [];
	};

	this.removeCartItem = function (cartItemCode) {
		var _this3 = this;

		this.cart.some(function (item, index) {
			if (item.CDPRODUTO === cartItemCode) {
				_this3.cart.splice(index, 1);
				return true;
			}
			return false;
		});
	};

	this.updateCart = function (cart) {
		this.cart = cart;
	};

	this.getFullComboCode = function (item) {
		var itemCode = item.CDPRODUTO;

		for (var i in item.comboCart) {
			item.comboCart[i].products.forEach(function (comboItem, index) {
				itemCode += comboItem.CDPRODUTO;
			});
		}

		return itemCode;
	};

	this.getProductObservations = function (item) {
		return item.personalizations.observations.filter(function (observation) {
			return observation.selected;
		});
	};

	this.getTotalCurrentComboProducts = function (products) {
		var total = 0;
		products.forEach(function (product) {
			total += product.QUANTIDADE;
		});
		return total;
	};

	var _getExistentProductInCart = function _getExistentProductInCart(cartItemCode, menuCart) {
		var productIndex = void 0;
		var exist = menuCart.some(function (item, index) {
			if (item.CDPRODUTO == cartItemCode) {
				productIndex = index;
				return true;
			}
		});

		return {
			'exist': exist,
			'productIndex': productIndex
		};
	};

	var _getExistentComboInCart = function _getExistentComboInCart(combo, menuCart) {
		var comboCode = self.getFullComboCode(combo);
		var productIndex = void 0,
		    itemCode = void 0;
		var exist = menuCart.some(function (item, index) {
			if (item.IDTIPOCOMPPROD == '3' || item.IDTIPOCOMPPROD == 'C') {
				itemCode = self.getFullComboCode(item);
				if (itemCode == comboCode) {
					productIndex = index;
					return true;
				}
			}
		});

		return {
			'exist': exist,
			'productIndex': productIndex
		};
	};

	var _isComboCartFilled = function _isComboCartFilled(comboCart) {
		var isFilled = true;
		Object.keys(comboCart).forEach(function (cartKey) {
			var currentCombo = comboCart[cartKey];
			if (self.getTotalCurrentComboProducts(currentCombo.products) < currentCombo.QTPRGRUPPROMOC) {
				isFilled = false;
			}
		});
		return isFilled;
	};
}
Configuration(function (ContextRegister) {
	ContextRegister.register('MenuService', MenuService);
});

// FILE: src/service/MenuUtil.js
function MenuUtil() {}

Array.prototype.first = function () {
	return this[0];
};

// FILE: src/controller/ComboProductDetailController.js
function ComboProductDetailController($scope, ScreenService) {
	var self = this;
	var widget = $scope.widget;
	self.product = widget.product;

	this.decreaseQuantity = function () {
		if (self.product.QUANTIDADE > 1) {
			self.product.QUANTIDADE--;
		}
	};

	this.increaseQuantity = function () {
		if (self.product.QUANTIDADE < 10) {
			self.product.QUANTIDADE++;
		}
	};

	this.getComboProductPrice = function () {
		return this.product.QUANTIDADE * parseFloat(this.product.VRPRECITEM);
	};
}

// FILE: src/controller/MenuTemplateController.js
function MenuTemplateController($scope, eventAggregator, ScreenService, ApplicationContext) {

	var widget = $scope.widget;

	this.buildMenu = function () {
		_bindWatcherForProducts();
		_loadTemplateStyle();
	};

	var _bindWatcherForProducts = function _bindWatcherForProducts() {
		var menuData = [];
		$scope.$watch("widget.menuProducts", function () {
			if (widget.menuProducts) {
				Object.keys(widget.menuProducts).forEach(function (menuKey) {
					var currentMenu = widget.menuProducts[menuKey];

					if (!currentMenu.subgrupos) {
						currentMenu.subgrupos = [{
							CODIGO: currentMenu.CODIGO,
							DESC: currentMenu.DESC,
							products: []
						}];

						var menuProducts = [];
						Object.keys(currentMenu.produtos).forEach(function (productKey) {
							var currentProduct = currentMenu.produtos[productKey];
							if (currentProduct.IDTIPOCOMPPROD !== 'C') {
								currentProduct.personalizations = _buildProductPersonalizations(currentProduct.OBSERVACOES);
							} else {
								for (var i in currentProduct.GRUPOS.produtos) {
									currentProduct.GRUPOS.produtos[i].personalizations = _buildProductPersonalizations(currentProduct.GRUPOS.produtos[i].OBSERVACOES);
								}
							}
							menuProducts.push(currentProduct);
						});

						currentMenu.subgrupos[0].products = menuProducts;
					}

					// Filter groups to don't repeat them on menu data
					var groupIndex = menuData.findIndex(function (menuGroup) {
						return menuGroup.grupo.CODIGO == currentMenu.grupo.CODIGO;
					});

					if (groupIndex === -1) {
						menuData.push(currentMenu);
					} else {
						menuData[groupIndex] = currentMenu;
					}
				});
			}
			widget.dataSource.data = menuData;
		});
	};

	var _loadTemplateStyle = function _loadTemplateStyle() {
		$scope.templateColors = {
			group: {
				'color': widget.menuProperties.colors.group.text,
				'background-color': widget.menuProperties.colors.group.background
			},
			openedGroup: {
				'color': widget.menuProperties.colors.group.opened.text,
				'background-color': widget.menuProperties.colors.group.opened.background,
				'border-left': "3px solid " + widget.menuProperties.colors.group.opened.text
			}
		};
	};

	var _buildProductPersonalizations = function _buildProductPersonalizations(personalizationsArray) {
		var productPersonalizations = {
			ingredients: [],
			observations: [],
			customObservation: ''
		};

		if (widget.personalizations.length) {
			personalizationsArray.forEach(function (personalizationCode) {
				var observation = widget.personalizations.filter(function (observation) {
					observation.selected = false;
					return observation.CDOCORR == personalizationCode;
				}).shift();

				if (observation.IDCONTROLAOBS == 'A') {
					if (observation.CDPRODUTO) {
						if (parseFloat(observation.VRPRECITEM) === 0) {
							observation.VRPRECITEM = 0.1;
						}
						var _ingredient = angular.copy(observation);
						productPersonalizations.ingredients.push(_ingredient);
					}
				} else {
					var _observation = angular.copy(observation);
					productPersonalizations.observations.push(_observation);
				}
			});
		}
		return productPersonalizations;
	};

	this.collapseProducts = function (group) {
		if (group.opened) {
			group.opened = !group.opened;
		} else {
			group.opened = true;
		}
	};

	// Select product PopUp
	this.selectProduct = function (product) {
		var template = void 0,
		    showAsModal = void 0;
		if (product.IDTIPOCOMPPROD == '3' || product.IDTIPOCOMPPROD == 'C') {
			template = 'odh-menu#combo-menu.html';
			showAsModal = true;
			product.menuGroups = _getBuiltComboGroups(product);
		} else {
			template = 'odh-menu#product-detail.html';
			showAsModal = false;
			product.menuGroups = [];
		}

		var selectedProductActions = [{
			"label": "Voltar",
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup();"
			}]
		}];

		var popupWidget = metaDataFactory.widgetFactory({
			"name": "selectedProduct",
			"container": widget.container,
			"parentWidget": widget,
			"label": "Detalhes",
			"template": template,
			"showAsModal": showAsModal,
			"isVisible": true,
			"popupNoBlock": false,
			"actions": selectedProductActions,
			"product": product,
			"menuProperties": widget.menuProperties
		});

		// Set the selectedProduct on parent widget
		_setSelectedProductWidget(popupWidget);

		product.QUANTIDADE = 1;
		ScreenService.openPopup(popupWidget);
		ApplicationContext.MenuService.selectedProduct = product;
	};

	var _setSelectedProductWidget = function _setSelectedProductWidget(popupWidget) {
		widget.widgets = [popupWidget];
	};

	var _getBuiltComboGroups = function _getBuiltComboGroups(product) {
		var _groups = [];
		if (!product.comboCart) {
			product.comboCart = {};
		}
		Object.keys(product.GRUPOS).forEach(function (groupKey, idx, groups) {
			var group = product.GRUPOS[groupKey].grupo;
			var products = [];

			Object.keys(product.GRUPOS[groupKey].produtos).forEach(function (productKey) {
				var currentProduct = product.GRUPOS[groupKey].produtos[productKey];
				currentProduct.personalizations = _buildProductPersonalizations(currentProduct.OBSERVACOES);
				currentProduct.parentProduct = product.CDPRODUTO;
				currentProduct.parentComboItem = group.CDGRUPROMOC;
				products.push(currentProduct);
			});

			product.comboCart[group.CDGRUPROMOC] = {
				'QTPRGRUPPROMOC': group.QTPRGRUPPROMOC,
				'products': []
			};

			_groups.push({
				'grupo': group,
				'products': products,
				'opened': groups.length === 1
			});
		});

		return _groups;
	};

	this.getRemainingProductsMessage = function (group) {
		var comboCart = widget.product.comboCart[group.grupo.CDGRUPROMOC];

		var totalComboLength = comboCart.QTPRGRUPPROMOC;
		//var comboCartLength = ApplicationContext.MenuService.getTotalCurrentComboProducts(comboCart.products);
		return totalComboLength
	};

	this.getRemainingProductsLength = function (group) {
		var comboCart = widget.product.comboCart[group.grupo.CDGRUPROMOC];
		return comboCart.QTPRGRUPPROMOC - ApplicationContext.MenuService.getTotalCurrentComboProducts(comboCart.products);
	};

	this.getProductsGroupQuantity = function (group) {
		var cart = ApplicationContext.MenuService.getCart();
		return cart.filter(function (item) {
			return item.NMGRUPO === group.DESC;
		}).length;
	};

	this.showSelectedProductLabel = function (product) {
		var cart = ApplicationContext.MenuService.getCart();
		return cart.some(function (cartItem) {
			return cartItem.CDPRODUTO === product.CDPRODUTO;
		});
	};

	this.showSelectedComboProductLabel = function (group, product) {
		var comboCart = widget.product.comboCart[group.CDGRUPROMOC];
		var productSelected = comboCart.products.find(function (cartItem) {
			return cartItem.CDPRODUTO === product.CDPRODUTO;
		});
		return !!productSelected;
	};
}

// FILE: src/controller/PersonalizeObservationsController.js
function PersonalizeObservationsController($scope, eventAggregator, ScreenService) {

	var self = this;
	var widget = $scope.widget;
	var personalizations = widget.personalizations;

	this.personalizationOptionClick = function (personalization) {
		personalization.selected = !personalization.selected;
	};

	this.clearCustomObservation = function () {
		widget.parentWidget.product.personalizations.customObservation = "";
	};
}

// FILE: src/controller/ProductDetailController.js
function ProductDetailController($scope, $sce, $sceDelegate, ScreenService) {
	var self = this;
	var widget = $scope.widget;
	self.product = widget.product;

	this.decreaseQuantity = function () {
		if (self.product.QUANTIDADE > 1) {
			self.product.QUANTIDADE--;
		}
	};

	this.increaseQuantity = function () {
		if (self.product.QUANTIDADE < 10) {
			self.product.QUANTIDADE++;
		}
	};

	this.openPeronalizationsModal = function (modalName, modalTemplate, personalizations) {
		var personalizeModalActions = [{
			"label": "Voltar",
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup();"
			}]
		}, {
			"label": "Salvar",
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup();"
			}]
		}];

		var popupWidget = metaDataFactory.widgetFactory({
			"name": "personalizeModal",
			"container": widget.container,
			"parentWidget": widget,
			"label": modalName,
			"showAsModal": true,
			"template": "odh-menu#" + modalTemplate + ".html",
			"isVisible": true,
			"popupNoBlock": false,
			"actions": personalizeModalActions,
			"personalizations": personalizations ? personalizations : [],
			"showFooter": false
		});

		ScreenService.openPopup(popupWidget);
	};

	this.getTotalProductPrice = function () {
		return this.product.QUANTIDADE * parseFloat(this.product.VRPRECITEM);
	};

	this.showPersonalizations = function () {
		return self.product.personalizations.ingredients.length > 0 || self.product.personalizations.observations.length > 0 || widget.parentWidget.showCustomObservation;
	};

	$scope.loadProductImg = function () {
		return $sce.trustAsResourceUrl(self.product.DSENDEIMG);
	};
}
//# sourceMappingURL=odh-menu.js.map
	